<?php

/**
 * @file
 * Administration functionality for the PoolParty GraphSearch.
 */

/******************************************************
 * Configuration set functions
 *****************************************************/

/**
 * List all saved PoolParty GraphSearch configurations.
 *
 * @return array
 *   The renderable HTML of the list of configurations.
 */
function sonr_webmining_configuration_list() {
  $output = array();

  $output['sonr_webmining_title'] = array(
    '#markup' => '<h3 class="semantic-connector-table-title">' . t('PoolParty GraphSearch configurations') . '</h3>',
  );

  $rows = array();
  $config_sets = sonr_webmining_config_load_multiple();
  /** @var \SonrWebminingConfigurationSet $config */
  foreach ($config_sets as $config) {
    $path = sonr_webmining_get_page_path($config->getId());
    /** @var \SemanticConnectorPPServerConnection $connection */
    $connection = $config->getConnection();
    $path = empty($path) ? '<div class="semantic-connector-italic">' . t('not yet set') . '</div>' : l('/' . $path, $path);
    $actions = array(
      l(t('Edit'), 'admin/config/semantic-drupal/sonr-webmining/' . $config->getId()),
      l(t('Clone'), 'admin/config/semantic-drupal/sonr-webmining/' . $config->getId() . '/clone'),
      l(t('Delete'), 'admin/config/semantic-drupal/sonr-webmining/' . $config->getId() . '/delete'),
    );

    // Get the search space label.
    $search_space_id = $config->getSearchSpaceId();
    $connection_config = $connection->getConfig();
    $sonr_config = $connection_config['sonr_configuration'];
    $search_space_label = 'search space not found';
    if (is_array($sonr_config) && !empty($sonr_config['projects'])) {
      if (version_compare($sonr_config['version'], '6.1', '>=')) {
        $search_spaces = semantic_connector_get_graphsearch_search_spaces($sonr_config);
        foreach ($search_spaces as $search_space) {
          if ($search_space['id'] == $search_space_id) {
            $search_space_label = $search_space['name'];
            break;
          }
        }
      }
      else {
        $projects = $connection->getApi('PPT')->getProjects();
        foreach ($projects as $project) {
          if (isset($sonr_config['projects'][$project->id]) && $project->id == $search_space_id) {
            $search_space_label = $project->title;
            break;
          }
        }
      }
    }

    $title = '<div class="semantic-connector-led" data-server-id="' . $connection->getId() . '" data-server-type="pp-server" title="' . t('Checking service') . '"></div>';
    $title .= $config->getTitle();
    $rows[] = array(
      $title,
      l($connection->getTitle(), $connection->getUrl() . '/' . $connection->getGraphSearchPath(), array('attributes' => array('target' => '_blank'))),
      $search_space_label,
      $path,
      implode(' | ', $actions),
    );
  }

  $output['sonr_webmining'] = array(
    '#theme' => 'table',
    '#header' => array(
      t('Name'),
      t('PoolParty GraphSearch server'),
      t('Selected search space'),
      t('Available on page'),
      t('Operations'),
    ),
    '#rows' => $rows,
    '#attributes' => array(
      'id' => 'sonr-webmining-configurations-table',
      'class' => array('semantic-connector-tablesorter'),
    ),
    '#attached' => array(
      'js' => array(drupal_get_path('module', 'sonr_webmining') . '/js/sonr_webmining.admin.js'),
      'css' => array(drupal_get_path('module', 'sonr_webmining') . '/css/sonr_webmining.admin.css'),
    ),
  );

  // Add the tablesorter library if available.
  $tablesorter_path = 'sites/all/libraries/tablesorter';
  if (module_exists('libraries')) {
    $tablesorter_path = libraries_get_path('tablesorter');
  }
  if (file_exists($tablesorter_path . '/jquery.tablesorter.min.js')) {
    $output['#attached']['js'][] = libraries_get_path('tablesorter') . '/jquery.tablesorter.min.js';
  }

  return $output;
}

/**
 * Ajax callback function for checking if a new PoolParty GraphSearch server
 * is available.
 */
function sonr_webmining_new_available($form, $form_state) {
  $available = '<div id="health_info" class="available"><div class="semantic-connector-led led-green" title="Service available"></div>' . t('The server is available.') . '</div>';
  $not_available = '<div id="health_info" class="not-available"><div class="semantic-connector-led led-red" title="Service NOT available"></div>' . t('The server is not available or the credentials are incorrect.') . '</div>';

  if (isset($form_state['values']['url']) && valid_url($form_state['values']['url'], TRUE)) {
    // Search for already existing connections.
    // (added to make overrides work here).
    $potential_connections = SemanticConnector::searchConnections(array(
      'type' => 'pp_server',
      'url' => $form_state['values']['url'],
    ));

    // Connection already exists, use the first one found.
    if (!empty($potential_connections)) {
      $connection = reset($potential_connections);
    }
    else {
      $connection = SemanticConnector::getConnection('pp_server');
      $connection->setUrl($form_state['values']['url']);
    }
    $connection->setCredentials(array(
      'username' => $form_state['values']['name'],
      'password' => $form_state['values']['pass'],
    ));

    $availability = $connection->getApi('PPX')->available();
    if (isset($availability['message']) && !empty($availability['message'])) {
      return '<div id="health_info" class="not-available"><div class="semantic-connector-led led-red" title="Service NOT available"></div>' . $availability['message'] . '</div>';
    }
    else {
      return $availability['success'] ? $available : $not_available;
    }
  }

  return $not_available;
}

/**
 * A form to update the PP server connection of a PoolParty GraphSearch
 * configuration.
 *
 * This form is used when creating a completely new PoolParty GraphSearch
 * configuration or when the PoolParty server connection needs to be changed or
 * a different project shell be used for an existing PoolParty GraphSearch
 * configuration.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param object $config
 *   A SonrWebminingConfigurationSet-instance
 *
 * @return array
 *   The Drupal form array.
 */
function sonr_webmining_connection_form($form, &$form_state, $config = NULL) {
  if (!is_null($config) && !empty($config)) {
    $is_configuration_new = FALSE;
    $form['swid'] = array(
      '#type' => 'hidden',
      '#value' => $config->getId(),
    );

    $form['title'] = array(
      '#type' => 'hidden',
      '#value' => $config->getTitle(),
    );
  }
  else {
    $is_configuration_new = TRUE;
    $config = new SonrWebminingConfigurationSet();

    $form['title'] = array(
      '#type' => 'textfield',
      '#title' => t('Name'),
      '#description' => t('Name of the PoolParty GraphSearch configuration.'),
      '#size' => 35,
      '#maxlength' => 255,
      '#default_value' => $config->getTitle(),
      '#required' => TRUE,
      '#validated' => TRUE,
    );
  }

  $connection_overrides = variable_get('sonr_webmining_override_connections', array());
  $overridden_values = array();
  if (!$is_configuration_new && isset($connection_overrides[$config->getId()])) {
    $overridden_values = $connection_overrides[$config->getId()];
  }

  // Container: Server settings.
  $form['server_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('1. Select the PoolParty server to use'),
  );

  if (isset($overridden_values['connection_id'])) {
    $form['server_settings']['overridden_connection'] = array(
      '#type' => 'markup',
      '#markup' => '<span class="semantic-connector-overridden-value">' . t('Warning: overridden by variable') . '</span>',
    );
  }

  $connections = SemanticConnector::getConnectionsByType('pp_server');
  if (!empty($connections)) {
    $connection_options = array();
    /** @var \SemanticConnectorPPServerConnection $connection */
    foreach ($connections as $connection) {
      $connection_config = $connection->getConfig();
      if (!empty($connection_config['sonr_configuration'])) {
        $credentials = $connection->getCredentials();
        $connection_options[implode('|', array(
          $connection->getTitle(),
          $connection->getUrl(),
          $credentials['username'],
          $credentials['password']
        ))] = $connection->getTitle();
      }
    }
    if (!empty($connection_options)) {
      $form['server_settings']['load_connection'] = array(
        '#type' => 'select',
        '#title' => t('Load an available PoolParty server'),
        '#options' => $connection_options,
        '#empty_option' => '',
        '#default_value' => '',
      );
    }
  }

  // Container: Connection details.
  $form['server_settings']['connection_details'] = array(
    '#type' => 'fieldset',
    '#title' => t('Connection details'),
  );

  $connection = $config->getConnection();
  $form['server_settings']['connection_details']['connection_id'] = array(
    '#type' => 'hidden',
    '#value' => $connection->getId(),
  );

  $form['server_settings']['connection_details']['server_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Server title'),
    '#description' => t('A short title for the server below.'),
    '#size' => 35,
    '#maxlength' => 60,
    '#default_value' => $connection->getTitle(),
    '#required' => TRUE,
  );

  $form['server_settings']['connection_details']['url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#description' => t('URL, where the PoolParty server runs.'),
    '#size' => 35,
    '#maxlength' => 255,
    '#default_value' => $connection->getUrl(),
    '#required' => TRUE,
  );

  $credentials = $connection->getCredentials();
  $form['server_settings']['connection_details']['credentials'] = array(
    '#type' => 'fieldset',
    '#title' => t('Credentials'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['server_settings']['connection_details']['credentials']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#description' => t('Name of a user for the credentials.'),
    '#size' => 35,
    '#maxlength' => 60,
    '#default_value' => $credentials['username'],
  );
  $form['server_settings']['connection_details']['credentials']['pass'] = array(
    '#type' => 'textfield',
    '#title' => t('Password'),
    '#description' => t('Password of a user for the credentials.'),
    '#size' => 35,
    '#maxlength' => 128,
    '#default_value' => $credentials['password'],
  );

  $form['server_settings']['health_check'] = array(
    '#type' => 'button',
    '#value' => t('Health check'),
    '#ajax' => array(
      'callback' => 'sonr_webmining_new_available',
      'wrapper' => 'health_info',
      'method' => 'replace',
      'effect' => 'slide',
    ),
  );

  if ($is_configuration_new) {
    $markup = '<div id="health_info">' . t('Click to check if the server is available.') . '</div>';
  }
  else {
    $available = '<div id="health_info" class="available"><div class="semantic-connector-led led-green" title="Service available"></div>' . t('The server is available.') . '</div>';
    $not_available = '<div id="health_info" class="not-available"><div class="semantic-connector-led led-red" title="Service NOT available"></div>' . t('The server is not available or the credentials are incorrect.') . '</div>';
    $markup = $connection->available() ? $available : $not_available;
  }
  $form['server_settings']['health_info'] = array(
    '#markup' => $markup,
  );

  // Container: Search space loading.
  $form['search_space_load'] = array(
    '#type' => 'fieldset',
    '#title' => t('2. Load the search spaces'),
  );

  $form['search_space_load']['load_search_spaces'] = array(
    '#type' => 'button',
    '#value' => 'Load search spaces',
    '#ajax' => array(
      'event' => 'click',
      'callback' => 'sonr_webmining_connection_form_get_search_spaces',
      'wrapper' => 'search_spaces-replace',
    ),
  );

  // Container: Search Space selection.
  $form['search_space_select'] = array(
    '#type' => 'fieldset',
    '#title' => t('3. Select the search space to use'),
    '#description' => t('Note: In case this list is still empty after clicking the "Load search spaces" button make sure that a connection to the PoolParty server can be established and check the rights of your selected user inside PoolParty.'),
  );

  // Get the search space options for the currently configured PoolParty server.
  $search_space_options = array();
  if (!$is_configuration_new) {
    $connection_config = $connection->getConfig();
    $sonr_config = $connection_config['sonr_configuration'];
    if (is_array($sonr_config)) {
      if (is_array($sonr_config)) {
        if (version_compare($sonr_config['version'], '6.1', '>=')) {
          $search_spaces = semantic_connector_get_graphsearch_search_spaces($sonr_config);
          foreach ($search_spaces as $search_space) {
            $search_space_options[$search_space['id']] = $search_space['name'] . ' (' . $search_space['language'] . ')';
          }
        }
        else {
          $projects = $connection->getApi('PPT')->getProjects();
          foreach ($projects as $project) {
            if (is_array($sonr_config) && isset($sonr_config['projects'][$project->id])) {
              $search_space_options[$project->id] = $project->title . ' (' . $sonr_config['projects'][$project->id]['search_spaces'][$project->id]['language'] . ')';
            }
          }
        }
      }
    }
  }
  // configuration set admin page.
  $form['search_space_select']['search_space'] = array(
    '#type' => 'select',
    '#title' => 'Select a search space',
    '#prefix' => '<div id="search_spaces-replace">',
    '#suffix' => '</div>',
    '#options' => $search_space_options,
    '#default_value' => (!$is_configuration_new ? $config->getSearchSpaceId() : NULL),
    '#required' => TRUE,
    '#validated' => TRUE,
  );

  if (isset($overridden_values['project_id'])) {
    $form['search_space_select']['search_space']['#description'] = '<span class="semantic-connector-overridden-value">' . t('Warning: overridden by variable') . '</span>';
  }

  // Save and cancel buttons.
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => ($is_configuration_new ? t('Save and edit details') : t('Save')),
    '#prefix' => '<div class="admin-form-submit-buttons" style="margin-top: 40px;">',
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/semantic-drupal/sonr-webmining',
    '#suffix' => '</div>',
  );

  // Add CSS and JS files
  $form['#attached'] = array(
    'js' => array(drupal_get_path('module', 'sonr_webmining') . '/js/sonr_webmining.admin.js'),
    'css' => array(drupal_get_path('module', 'sonr_webmining') . '/css/sonr_webmining.admin.css'),
  );

  return $form;
}

/**
 * Ajax callback function to get a search space select list (or project list for
 * older GraphSearches) for a given PoolParty server connection configuration.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 *
 * @return array
 *   The select form element containing the search space options for the current
 *   PoolParty server connection.
 */
function sonr_webmining_connection_form_get_search_spaces($form, $form_state) {
  $search_space_element = $form['search_space_select']['search_space'];

  $search_space_options = array();
  if (isset($form_state['values']['url']) && valid_url($form_state['values']['url'], TRUE)) {
    // Search for already existing connections.
    // (added to make overrides work here).
    $potential_connections = SemanticConnector::searchConnections(array(
      'type' => 'pp_server',
      'url' => $form_state['values']['url'],
    ));

    // Connection already exists, use the first one found.
    if (!empty($potential_connections)) {
      $connection = reset($potential_connections);
    }
    else {
      $connection = SemanticConnector::getConnection('pp_server');
      $connection->setUrl($form_state['values']['url']);
    }

    $connection->setCredentials(array(
      'username' => $form_state['values']['name'],
      'password' => $form_state['values']['pass'],
    ));
    $sonr_config = $connection->getApi('sonr')->getConfig();
    $connection_config = $connection->getConfig();
    if (is_array($sonr_config)) {
      if (version_compare($connection_config['sonr_configuration']['version'], '6.1', '>=')) {
        $search_spaces = semantic_connector_get_graphsearch_search_spaces($sonr_config);
        foreach ($search_spaces as $search_space) {
          $search_space_options[$search_space['id']] = $search_space['name'] . ' (' . $search_space['language'] . ')';
        }
      }
      else {
        $projects = $connection->getApi('PPT')->getProjects();
        foreach ($projects as $project) {
          if (is_array($sonr_config) && isset($sonr_config['projects'][$project->id])) {
            $search_space_options[$project->id] = $project->title . ' (' . $sonr_config['projects'][$project->id]['search_spaces'][$project->id]['language'] . ')';
          }
        }
      }
    }
  }

  $search_space_element['#options'] = $search_space_options;
  return $search_space_element;
}

/**
 * The validation handler for the PoolParty GraphSearch connection form.
 */
function sonr_webmining_connection_form_validate($form, &$form_state) {
  // Only do search space validation during the save-operation, not during
  // ajax-requests like the health check of the server.
  if ($form_state['triggering_element']['#parents'][0] == 'save') {
    // A title is required.
    if (!isset($form_state['values']['swid']) && empty($form_state['values']['title'])) {
      form_set_error('title', t('Name field is required.'));
    }

    // A search space needs to be selected.
    if (empty($form_state['values']['search_space'])) {
      form_set_error('search_space', t('Please select a search space.'));
    }
    // And it has to be a valid search space (one that is connected to a
    // PoolParty GraphSearch server).
    else {
      if (isset($form_state['values']['url']) && valid_url($form_state['values']['url'], TRUE)) {
        // Search for already existing connections.
        // (added to make overrides work here).
        $potential_connections = SemanticConnector::searchConnections(array(
          'type' => 'pp_server',
          'url' => $form_state['values']['url'],
        ));

        // Connection already exists, use the first one found.
        if (!empty($potential_connections)) {
          $connection = reset($potential_connections);
        }
        else {
          $connection = SemanticConnector::getConnection('pp_server');
          $connection->setUrl($form_state['values']['url']);
        }
        $connection->setCredentials(array(
          'username' => $form_state['values']['name'],
          'password' => $form_state['values']['pass'],
        ));
        $sonr_config = $connection->getApi('sonr')->getConfig();
        $connection_config = $connection->getConfig();
        if (version_compare($connection_config['sonr_configuration']['version'], '6.1', '>=')) {
          $search_spaces = semantic_connector_get_graphsearch_search_spaces($sonr_config);
          if (!isset($search_spaces[$form_state['values']['search_space']])) {
            form_set_error('search_space', t('There is no PoolParty GraphSearch server available for the selected search space.'));
          }
        }
        else {
          if (!is_array($sonr_config) || !isset($sonr_config['projects'][$form_state['values']['search_space']])) {
            form_set_error('search_space', t('There is no PoolParty GraphSearch server available for the selected search space.'));
          }
        }
      }
      else {
        form_set_error('url', t('The field URL must be a valid URL.'));
      }
    }
  }
}

/**
 * The submit handler for the PoolParty GraphSearch connection form.
 */
function sonr_webmining_connection_form_submit($form, &$form_state) {
  // Always create a new connection, if URL and type are the same the old one
  // will be used anyway.
  $connection = SemanticConnector::createConnection('pp_server', $form_state['values']['url'], $form_state['values']['server_title'], array(
    'username' => $form_state['values']['name'],
    'password' => $form_state['values']['pass'],
  ));

  // Update an existing configuration set.
  if (isset($form_state['values']['swid'])) {
    $swid = $form_state['values']['swid'];
    db_update('sonr_webmining_sets')
      ->fields(array(
        'connection_id' => $connection->getId(),
        'search_space_id' => $form_state['values']['search_space'],
      ))
      ->condition('swid', $swid)
      ->execute();

    drupal_set_message(t('The connection for PoolParty GraphSearch configuration-set %title has been updated.', array('%title' => $form_state['values']['title'])));
  }
  // Create a new configuration set.
  else {
    $swid = db_insert('sonr_webmining_sets')
      ->fields(array(
        'title' => $form_state['values']['title'],
        'connection_id' => $connection->getId(),
        'search_space_id' => $form_state['values']['search_space'],
      ))
      ->execute();

    // Add all facets once the set is created.
    $aggregator = new SonrWebmining($swid);
    $config['facets_to_show'] = array_keys($aggregator->getAllFacets());
    db_update('sonr_webmining_sets')
      ->fields(array(
        'config' => serialize(SonrWebminingConfigurationSet::getDefaultConfig()),
      ))
      ->condition('swid', $swid)
      ->execute();

    drupal_set_message(t('PoolParty GraphSearch configuration-set %title has been created.', array('%title' => $form_state['values']['title'])));
  }

  $form_state['redirect'] = 'admin/config/semantic-drupal/sonr-webmining/' . $swid;
}

/**
 * A form to edit an existing PoolParty GraphSearch configuration.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param object $config
 *   A SonrWebminingConfigurationSet-instance
 *
 * @return array
 *   The Drupal form array.
 */
function sonr_webmining_configuration_set_form($form, &$form_state, $config) {
  $form['swid'] = array(
    '#type' => 'hidden',
    '#value' => $config->getId(),
  );
  $configuration = $config->getConfig();

  $connection_overrides = variable_get('sonr_webmining_override_connections', array());
  $overridden_values = array();
  if (isset($connection_overrides[$config->getId()])) {
    $overridden_values = $connection_overrides[$config->getId()];
  }

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('Name of the PoolParty GraphSearch configuration.'). (isset($overridden_values['title']) ? ' <span class="semantic-connector-overridden-value">' . t('Warning: overridden by variable') . '</span>' : ''),
    '#size' => 35,
    '#maxlength' => 255,
    '#default_value' => $config->getTitle(),
    '#required' => TRUE,
  );

  /** @var \SemanticConnectorPPServerConnection $connection */
  $connection = $config->getConnection();
  // Get the search space label.
  $search_space_id = $config->getSearchSpaceId();
  $connection_config = $connection->getConfig();
  $sonr_config = $connection_config['sonr_configuration'];
  $search_space_label = t('<invalid search space selected>');
  if (is_array($sonr_config)) {
    if (is_array($sonr_config)) {
      if (version_compare($sonr_config['version'], '6.1', '>=')) {
        $search_spaces = semantic_connector_get_graphsearch_search_spaces($sonr_config);
        foreach ($search_spaces as $search_space) {
          if ($search_space['id'] == $search_space_id) {
            $search_space_label = $search_space['name'];
            break;
          }
        }
      }
      else {
        $projects = $connection->getApi('PPT')->getProjects();
        foreach ($projects as $project) {
          if (isset($sonr_config['projects'][$project->id]) && $project->id == $search_space_id) {
            $search_space_label = $project->title;
            break;
          }
        }
      }
    }
  }

  // Add information about the connection.
  $connection_markup = '';
  // Check the PoolParty server version if required.
  if (variable_get('semantic_connector_version_checking', TRUE)) {
    $version_messages = array();

    $ppx_api_version_info = $connection->getVersionInfo('PPX');
    if (version_compare($ppx_api_version_info['installed_version'], $ppx_api_version_info['latest_version'], '<')) {
      $version_messages[] = t('The connected PoolParty server is not up to date. You are currently running version %installedversion, upgrade to version %latestversion to enjoy the new features.', array('%installedversion' => $ppx_api_version_info['installed_version'], '%latestversion' => $ppx_api_version_info['latest_version']));
    }

    $sonr_api_version_info = $connection->getVersionInfo('sonr');
    if (version_compare($sonr_api_version_info['installed_version'], $sonr_api_version_info['latest_version'], '<')) {
      $version_messages[] = t('The connected PoolParty GraphSearch server is not up to date. You are currently running version %installedversion, upgrade to version %latestversion to enjoy the new features.', array('%installedversion' => $sonr_api_version_info['installed_version'], '%latestversion' => $sonr_api_version_info['latest_version']));
    }

    if (!empty($version_messages)) {
      $connection_markup .= '<div class="messages warning"><div class="message">' . implode('</div><div class="message">', $version_messages) . '</div></div>';
    }
  }
  $connection_markup .= '<p id="sonr-webmining-connection-info">' . t('Connected PoolParty server') . ': <b>' . $connection->getTitle() . ' (' . $connection->getUrl() . ')</b><br />'
    . t('Selected search space') . ': <b>' . $search_space_label . '</b><br />'
    . l(t('Change the connected PoolParty server or search space'), 'admin/config/semantic-drupal/sonr-webmining/' . $config->getId() . '/edit_connection') . '</p>';
  $form['pp_connection_markup'] = array(
    '#type' => 'markup',
    '#markup' => $connection_markup,
  );

  // Define the container for the vertical tabs.
  $form['settings'] = array(
    '#type' => 'vertical_tabs',
  );

  // Tab: Result settings.
  $form['result_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Result settings'),
    '#group' => 'settings',
  );

  $form['result_settings']['items_per_page'] = array(
    '#type' => 'textfield',
    '#title' => t('Items per page'),
    '#description' => t('The number of items you want to display on the first result page.'),
    '#size' => 15,
    '#maxlength' => 5,
    '#default_value' => $configuration['items_per_page'],
    '#required' => TRUE,
    '#element_validate' => array('element_validate_integer_positive'),
  );

  $form['result_settings']['show_results_count'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show results count'),
    '#description' => t('This option adds a total count of results for the current selection above the results list.'),
    '#default_value' => $configuration['show_results_count'],
  );

  $form['result_settings']['result_row_settings'] = array(
    '#type' => 'fieldset',
    '#title' => 'Result item display',
  );

  $form['result_settings']['result_row_settings']['summary_max_chars'] = array(
    '#type' => 'textfield',
    '#title' => t('Length of the summary'),
    '#description' => t('The maximum length in characters you want to display on every result item.'),
    '#size' => 15,
    '#maxlength' => 5,
    '#default_value' => $configuration['summary_max_chars'],
    '#required' => TRUE,
    '#element_validate' => array('element_validate_integer_positive'),
  );

  $form['result_settings']['result_row_settings']['show_sentiment'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show dynamic sentiment'),
    '#description' => t('This option specifies whether dynamic sentiments should be displayed or not'),
    '#default_value' => $configuration['show_sentiment'],
  );

  $form['result_settings']['result_row_settings']['show_tags'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show a list of all tagged concepts for every content item.'),
    '#description' => t('This option specifies whether tags should be displayed or not'),
    '#default_value' => $configuration['show_tags'],
  );

  $form['result_settings']['result_row_settings']['tags_max_items_container'] = array(
    '#type' => 'container',
    '#attributes' => array('style' => array('padding-left:18px;')),
  );

  $form['result_settings']['result_row_settings']['tags_max_items_container']['tags_max_items'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of tags'),
    '#description' => t('The maximum number of tags you want to display. Leave empty to display all tags.'),
    '#size' => 15,
    '#maxlength' => 3,
    '#default_value' => $configuration['tags_max_items'],
    '#element_validate' => array('element_validate_integer_positive'),
    '#states' => array(
      'visible' => array(':input[name="show_tags"]' => array('checked' => TRUE)),
    ),
  );

  $form['result_settings']['result_row_settings']['show_similar'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show similar documents'),
    '#description' => t('This specifies whether the more like this option should be displayed or not'),
    '#default_value' => $configuration['show_similar'],
  );

  $form['result_settings']['result_row_settings']['similar_max_items_container'] = array(
    '#type' => 'container',
    '#attributes' => array('style' => array('padding-left:18px;')),
  );

  $form['result_settings']['result_row_settings']['similar_max_items_container']['similar_max_items'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of documents'),
    '#description' => t('The maximum number of similar documents you want to display.'),
    '#size' => 15,
    '#maxlength' => 3,
    '#default_value' => $configuration['similar_max_items'],
    '#element_validate' => array('element_validate_integer_positive'),
    '#states' => array(
      'visible' => array(':input[name="show_similar"]' => array('checked' => TRUE)),
      'required' => array(':input[name="show_similar"]' => array('checked' => TRUE)),
    ),
  );

  $form['result_settings']['advanced_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $form['result_settings']['advanced_settings']['first_page_only'] = array(
    '#type' => 'checkbox',
    '#title' => t('First page only'),
    '#description' => t('This option disables infinite loading as well as any kind of pagination.'),
    '#default_value' => $configuration['first_page_only'],
  );

  $period = drupal_map_assoc(array(0, 60, 180, 300, 600, 900, 1800, 2700, 3600, 10800, 21600, 32400, 43200, 86400), 'format_interval');
  $period[0] = '<' . t('none') . '>';
  $form['result_settings']['advanced_settings']['cache_lifetime'] = array(
    '#type' => 'select',
    '#title' => t('Minimum cache lifetime for the first result page'),
    '#description' => t('Cached result pages will not be re-created until at least this much time has elapsed.'),
    '#options' => $period,
    '#default_value' => $configuration['cache_lifetime'],
  );

  // Tab: Filter settings.
  $form['filter_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filter settings'),
    '#group' => 'settings',
  );

  $form['filter_settings']['separate_blocks'] = array(
    '#type' => 'checkbox',
    '#title' => t('Separate blocks for filters and the result list'),
    '#description' => t('If this option is selected, two blocks will be created. This offers more flexibility in positioning the filters-area or even hiding the filters completely.'),
    '#default_value' => $configuration['separate_blocks'],
  );

  $aggregator = new SonrWebmining($config->getId());
  $all_facets = $aggregator->getAllFacets();
  $stored_facets = array();
  $facets = array();
  foreach ($configuration['facets_to_show'] as $weight => $facet) {
    // Manage new facet data.
    if (is_array($facet)) {
      // Ignore removed facets on PoolParty GraphSearch server.
      if (!isset($all_facets[$facet['facet_id']])) {
        continue;
      }
      $facets[$weight] = $facet['facet_id'];
      $stored_facets[$facet['facet_id']] = $facet;
    }
    // Manage old facet data.
    else {
      // Ignore removed facets on PoolParty GraphSearch server.
      if (!isset($all_facets[$facet])) {
        continue;
      }
      $facets[$weight] = $facet;
      $stored_facets[$facet] = array(
        'facet_id' => $facet,
        'name' => '',
        'selected' => TRUE,
        'facet_mode' => 'list',
        'max_items' => 10,
        'tree_depth' => 1,
      );
    }
  }
  if (is_array($all_facets)) {
    foreach (array_keys($all_facets) as $facet_id) {
      if (!in_array($facet_id, $facets)) {
        $facets[] = $facet_id;
        $stored_facets[$facet_id] = array(
          'facet_id' => $facet_id,
          'name' => $all_facets[$facet_id],
          'selected' => FALSE,
          'facet_mode' => 'list',
          'max_items' => 10,
          'tree_depth' => 1,
        );
      }
    }
  }
  foreach ($facets as $weight => $facet_id) {
    $form['filter_settings']['facets_to_show']['#tree'] = TRUE;
    $form['filter_settings']['facets_to_show'][$facet_id]['node'] = array(
      '#markup' => $all_facets[$facet_id],
    );
    // This field is invisible, but contains sort info (weights).
    $form['filter_settings']['facets_to_show'][$facet_id]['weight'] = array(
      '#type' => 'weight',
      // Weights from -255 to +255 are supported because of this delta.
      '#delta' => 255,
      '#title_display' => 'invisible',
      '#default_value' => $weight,
    );
    $form['filter_settings']['facets_to_show'][$facet_id]['name'] = array(
      '#type' => 'textfield',
      '#size' => 15,
      '#maxlength' => 255,
      '#default_value' => isset($stored_facets[$facet_id]['name']) ? $stored_facets[$facet_id]['name'] : $all_facets[$facet_id],
    );
    $form['filter_settings']['facets_to_show'][$facet_id]['selected'] = array(
      '#type' => 'checkbox',
      '#default_value' => $stored_facets[$facet_id]['selected'],
    );
    $form['filter_settings']['facets_to_show'][$facet_id]['facet_mode'] = array(
      '#type' => 'select',
      '#options' => array(
        'list' => t('flat'),
        'aggregate' => t('top concepts'),
        'tree' => t('hierarchical'),
      ),
      '#default_value' => $stored_facets[$facet_id]['facet_mode'],
    );
    $form['filter_settings']['facets_to_show'][$facet_id]['max_items'] = array(
      '#type' => 'textfield',
      '#element_validate' => array('element_validate_integer_positive'),
      '#size' => 5,
      '#maxlength' => 3,
      '#default_value' => $stored_facets[$facet_id]['max_items'],
    );
    $form['filter_settings']['facets_to_show'][$facet_id]['tree_depth'] = array(
      '#type' => 'select',
      '#options' => array(0, 1, 2, 3),
      '#default_value' => $stored_facets[$facet_id]['tree_depth'],
      '#states' => array(
        'enabled' => array(':input[name="facets_to_show[' . $facet_id . '][facet_mode]"]' => array('value' => 'tree')),
      ),
    );
  }

  $form['filter_settings']['hide_empty_facet'] = array(
    '#type' => 'checkbox',
    '#title' => t('Hide facets without documents'),
    '#description' => t('If it\'s checked, the facets without documents will be hidden, otherwise it will be shown greyed out.'),
    '#default_value' => $configuration['hide_empty_facet'],
  );

  $form['filter_settings']['time_filter'] = array(
    '#type' => 'select',
    '#title' => t('Date filter format'),
    '#options' => array(
      'from_to_textfields' => t('Separate textfields for "from" and "to" date values'),
      'range_selection' => t('The user can select a time range (last day, last month, ...)'),
    ),
    '#empty_option' => t('No date filter'),
    '#empty_value' => NULL,
    '#default_value' => $configuration['time_filter'],
  );

  $form['filter_settings']['time_filter_years'] = array(
    '#type' => 'textfield',
    '#title' => t('Date filter with years'),
    '#description' => t('The year must be less then @year. Leave empty if no years should be displayed in the filter.', array('@year' => date('Y'))),
    '#field_prefix' => t('Add years to the time range filter from this year to') . ' ',
    '#field_suffix' => t('(e.g. 2012)'),
    '#default_value' => $configuration['time_filter_years'],
    '#maxlength' => 4,
    '#attributes' => array('style' => array('width:50px')),
    '#element_validate' => array('element_validate_integer_positive'),
    '#states' => array(
      'visible' => array(':input[name="time_filter"]' => array('value' => 'range_selection')),
    ),
  );

  if (!module_exists('date_popup')) {
    $link = l('Date', 'https://www.drupal.org/project/date', array('attributes' => array('target' => '_blank')));
    $hidden = $configuration['time_filter'] == 'from_to_textfields' ? '' : 'element-hidden';
    $form['filter_settings']['date_info'] = array(
      '#markup' => t('Please install the module !date and enable Date and Date Popup to use this feature.', array('!date' => $link)),
      '#prefix' => '<div id="edit-date-info" class="form-item ' . $hidden . '"><label>' . t('The module Date Popup is missing') . '</label>',
      '#suffix' => '</div>',
    );
  }

  $form['filter_settings']['add_trends'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add Trends chart'),
    '#description' => t('Adds a chart showing the trends for the selected filter of the current search.'),
    '#default_value' => $configuration['add_trends'],
  );
  if (!module_exists('sonr_webmining_trends')) {
    $link = l('PoolParty GraphSearch Trends', 'admin/modules');
    $hidden = $configuration['add_trends'] ? '' : 'element-hidden';
    $form['filter_settings']['trends_info'] = array(
      '#markup' => t('Please enable the module !trends to use this feature.', array('!trends' => $link)),
      '#prefix' => '<div id="edit-trends-info" class="form-item ' . $hidden . '"><label>' . t('The module PoolParty GraphSearch Trends is missing') . '</label>',
      '#suffix' => '</div>',
    );
  }

  $components = array(
    'facets' => t('Facets filter'),
    'time' => t('Date filter'),
    'reset' => t('Reset button'),
  );
  if (module_exists('sonr_webmining_trends')) {
    $components['trends'] = t('Trends chart');
  }
  $weighted_components = $configuration['components_order'];
  foreach (array_keys($components) as $component_id) {
    if (!in_array($component_id, $weighted_components)) {
      $weighted_components[] = $component_id;
    }
  }

  foreach ($weighted_components as $weight => $component_id) {
    if (!isset($components[$component_id])) {
      continue;
    }
    $form['filter_settings']['components_order']['#tree'] = TRUE;
    $form['filter_settings']['components_order'][$component_id]['node'] = array(
      '#markup' => $components[$component_id],
    );
    // This field is invisible, but contains sort info (weights).
    $form['filter_settings']['components_order'][$component_id]['weight'] = array(
      '#type' => 'weight',
      '#title_display' => 'invisible',
      '#default_value' => $weight,
    );
  }

  // Tab: Search bar settings.
  $form['searchbar_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Search bar settings'),
    '#group' => 'settings',
  );

  $form['searchbar_settings']['show_searchbar'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show the search bar'),
    '#description' => t('Check it if the search bar with the buttons "Search" and "Reset" should be displayed over the result list.'),
    '#default_value' => $configuration['show_searchbar'],
  );

  $form['searchbar_settings']['show_block_searchbar'] = array(
    '#type' => 'checkbox',
    '#title' => t('Create an additional block with the search bar'),
    '#description' => t('This checkbox creates an additional block with the search bar, which can be added into e.g. the header area.'),
    '#default_value' => $configuration['show_block_searchbar'],
  );

  $form['searchbar_settings']['placeholder'] = array(
    '#type' => 'textfield',
    '#title' => t('Placeholder'),
    '#description' => t('The placeholder is displayed in the input field before the user enters a value.'),
    '#size' => 15,
    '#default_value' => $configuration['placeholder'],
  );

  $form['searchbar_settings']['ac_min_chars'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of characters'),
    '#description' => t('The minimum number of characters that must be entered before the auto completion is performed (must be greater than 1).'),
    '#size' => 15,
    '#maxlength' => 3,
    '#default_value' => $configuration['ac_min_chars'],
    '#element_validate' => array('element_validate_integer_positive'),
  );

  $form['searchbar_settings']['suggestions'] = array(
    '#type' => 'fieldset',
    '#title' => t('Suggestions'),
  );

  $form['searchbar_settings']['suggestions']['ac_max_suggestions'] = array(
    '#type' => 'textfield',
    '#title' => t('Count of suggestions'),
    '#description' => t('The maximum number of terms that should be displayed in the drop down menu.'),
    '#size' => 15,
    '#maxlength' => 3,
    '#default_value' => $configuration['ac_max_suggestions'],
    '#element_validate' => array('element_validate_integer_positive'),
  );

  $form['searchbar_settings']['suggestions']['ac_add_matching_label'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add the matching label to the to every suggestion in the drop down menu.'),
    '#default_value' => $configuration['ac_add_matching_label'],
  );

  $form['searchbar_settings']['suggestions']['ac_add_context'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add the context (label of the parent concept) to the to every suggestion in the drop down menu.'),
    '#default_value' => $configuration['ac_add_context'],
  );

  $form['searchbar_settings']['suggestions']['ac_add_facet_name'] = array(
    '#type' => 'checkbox',
    '#title' => t('Additionally add the name of the facet after the context.'),
    '#default_value' => $configuration['ac_add_facet_name'],
    '#states' => array(
      'visible' => array(':input[name="ac_add_context"]' => array('checked' => TRUE)),
    ),
  );

  $form['searchbar_settings']['advanced_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $form['searchbar_settings']['advanced_settings']['show_facetbox'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show the filter container'),
    '#description' => t('Check it if a container with all selected filters should be displayed beneath the search bar.'),
    '#default_value' => $configuration['show_facetbox'],
  );

  $options = array(
    'concept' => t('Only terms from the suggestions are possible (it searches only in the tagged terms).'),
    'concept free-term' => t('In addition to the suggestions, the entry of free terms is possible (it matches the tagged terms and searches in the content).'),
  );
  $form['searchbar_settings']['advanced_settings']['search_type'] = array(
    '#type' => 'radios',
    '#title' => t('Select the type of search'),
    '#description' => t('The first option minimizes an empty result list. The second option, on the other hand, gives an user the possibility to search in the content.'),
    '#options' => $options,
    '#default_value' => $configuration['search_type'],
  );

  // Tab: Other settings.
  $form['other_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Other settings'),
    '#group' => 'settings',
  );

  $form['other_settings']['use_css_file'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use CSS File'),
    '#description' => t('Check if you wish to include the module\'s CSS file.'),
    '#default_value' => $configuration['use_css_file'],
  );

  $form['other_settings']['add_rss_functionality'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add RSS button'),
    '#description' => t('Check if you wish to add the RSS button and its functionallity.'),
    '#default_value' => $configuration['add_rss_functionality'],
  );

  // Tab: Semantic module connection.
  $form['semantic_connection'] = array(
    '#type' => 'fieldset',
    '#title' => t('Semantic module connection'),
    '#group' => 'settings',
    '#tree' => TRUE,
  );

  $form['semantic_connection']['show_in_destinations'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show as concept destination'),
    '#description' => t('Add the content block of this PoolParty GraphSearch configuration to a list of destinations for a concept link if applicable.'),
    '#default_value' => ((isset($configuration['semantic_connection']) && isset($configuration['semantic_connection']['show_in_destinations'])) ? $configuration['semantic_connection']['show_in_destinations'] : TRUE),
  );

  // Save and cancel buttons.
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  $form['delete'] = array(
    '#type' => 'button',
    '#value' => t('Delete'),
    '#attributes' => array('onclick' => 'window.location.href = "' . url('admin/config/semantic-drupal/sonr-webmining/' . $config->getId() . '/delete') . '"; return false;'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => (isset($_GET['destination']) ? $_GET['destination'] : 'admin/config/semantic-drupal/sonr-webmining'),
  );

  // Add JavaScript
  drupal_add_js(drupal_get_path('module', 'sonr_webmining') . '/js/sonr_webmining.admin.js');

  return $form;
}

/**
 * Theme function for the configuration set admin form.
 *
 * @param array $variables
 *   An array of variables supported by the theme.
 *
 * @return string
 *   The rendered HTML content of the admin form.
 */
function theme_sonr_webmining_configuration_set_form($variables) {
  $form = $variables['form'];

  // If the facets are not only an empty array (on creation of a new
  // configuration set), create a sortable table out of the facet-list.
  if (!isset($form['filter_settings']['facets_to_show']['#value'])) {
    $rows = array();
    foreach (element_children($form['filter_settings']['facets_to_show']) as $facet_id) {
      $form['filter_settings']['facets_to_show'][$facet_id]['weight']['#attributes']['class'] = array('facets-to-show-order-weight');
      $rows[] = array(
        'data' => array(
          drupal_render($form['filter_settings']['facets_to_show'][$facet_id]['node']),
          drupal_render($form['filter_settings']['facets_to_show'][$facet_id]['selected']),
          drupal_render($form['filter_settings']['facets_to_show'][$facet_id]['name']),
          drupal_render($form['filter_settings']['facets_to_show'][$facet_id]['facet_mode']),
          drupal_render($form['filter_settings']['facets_to_show'][$facet_id]['max_items']),
          drupal_render($form['filter_settings']['facets_to_show'][$facet_id]['tree_depth']),
          drupal_render($form['filter_settings']['facets_to_show'][$facet_id]['weight']),
        ),
        'class' => array('draggable'),
      );
    }

    $header = array(t('Facet name'), t('Show'), t('Filter name'), t('Facet mode'), t('Maximum terms'), t('Hierarchical depth'), t('Weight'));
    $form['filter_settings']['facets_to_show'] = array(
      '#type' => 'item',
      '#title' => t('Facets to show'),
      '#description' => t('Choose the facets you want your users to see as filter-sets and their order.'),
      '#markup' => theme(
        'table',
        array(
          'header' => $header,
          'rows' => $rows,
          'attributes' => array('id' => 'facets-to-show-order'),
        )
      ),
    );

    drupal_add_tabledrag('facets-to-show-order', 'order', 'sibling', 'facets-to-show-order-weight');
  }

  $rows = array();
  foreach (element_children($form['filter_settings']['components_order']) as $component_id) {
    $form['filter_settings']['components_order'][$component_id]['weight']['#attributes']['class'] = array('components-order-order-weight');
    $rows[] = array(
      'data' => array(
        array('class' => array('slide-cross')),
        drupal_render($form['filter_settings']['components_order'][$component_id]['node']),
        drupal_render($form['filter_settings']['components_order'][$component_id]['weight']),
      ),
      'class' => array('draggable'),
    );
  }

  $header = array('Order', t('Component name'), t('Weight'));
  $form['filter_settings']['components_order'] = array(
    '#type' => 'item',
    '#title' => t('Order of the filter components'),
    '#description' => t('Drag and drop the handles of the filter components to set their order.'),
    '#markup' => theme(
      'table',
      array(
        'header' => $header,
        'rows' => $rows,
        'attributes' => array('id' => 'components-order-order'),
      )
    ),
  );

  drupal_add_tabledrag('components-order-order', 'order', 'sibling', 'components-order-order-weight');

  $output = drupal_render_children($form);
  return $output;
}

/**
 * The validation handler for the PoolParty GraphSearch configuration admin form.
 */
function sonr_webmining_configuration_set_form_validate($form, &$form_state) {
  if ($form_state['values']['ac_min_chars'] <= 1) {
    form_set_error('ac_min_chars', t('"%field" must be greater than 1.', array('%field' => t('Number of characters'))));
  }
  $year = date('Y');
  if (!empty($form_state['values']['time_filter_years']) && $form_state['values']['time_filter_years'] >= $year) {
    form_set_error('time_filter_years', t('"%field" must be less than %year.', array('%field' => t('Date filter with years'), '%year' => $year)));
  }
  if ($form_state['values']['show_similar'] && !$form_state['values']['similar_max_items']) {
    form_set_error('similar_max_items', t('"%field" can not be empty.', array('%field' => t('Number of documents'))));
  }
}

/**
 * The submit handler for the PoolParty GraphSearch configuration admin form.
 */
function sonr_webmining_configuration_set_form_submit($form, &$form_state) {
  $previous_configuration_set = new SonrWebminingConfigurationSet($form_state['values']['swid']);

  $facets_to_show = array();
  $facet_max_items = 10;
  foreach ($form_state['values']['facets_to_show'] as $facet_id => $facet) {
    $weight = $facet['weight'];
    unset($facet['weight']);
    $facet['facet_id'] = $facet_id;
    $facets_to_show[$weight] = $facet;
    if ($facet_max_items < $facet['max_items']) {
      $facet_max_items = $facet['max_items'];
    }
  }
  if (!empty($facets_to_show)) {
    ksort($facets_to_show);
    $facets_to_show = array_values($facets_to_show);
  }

  $components_order = array();
  foreach ($form_state['values']['components_order'] as $component_id => $component) {
    $components_order[$component['weight']] = $component_id;
  }
  if (!empty($components_order)) {
    ksort($components_order);
    $components_order = array_values($components_order);
  }

  $config = array(
    // Result settings.
    'items_per_page' => $form_state['values']['items_per_page'],
    'show_results_count' => $form_state['values']['show_results_count'],
    'summary_max_chars' => $form_state['values']['summary_max_chars'],
    'show_tags' => $form_state['values']['show_tags'],
    'tags_max_items' => $form_state['values']['tags_max_items'],
    'show_similar' => $form_state['values']['show_similar'],
    'similar_max_items' => $form_state['values']['similar_max_items'],
    'show_sentiment' => $form_state['values']['show_sentiment'],
    'first_page_only' => $form_state['values']['first_page_only'],
    'cache_lifetime' => $form_state['values']['cache_lifetime'],
    // Filter settings.
    'separate_blocks' => $form_state['values']['separate_blocks'],
    'facets_to_show' => $facets_to_show,
    'facet_max_items' => $facet_max_items,
    'hide_empty_facet' => $form_state['values']['hide_empty_facet'],
    'time_filter' => $form_state['values']['time_filter'],
    'time_filter_years' => $form_state['values']['time_filter_years'],
    'add_trends' => $form_state['values']['add_trends'],
    'components_order' => $components_order,
    // Search bar settings.
    'show_searchbar' => $form_state['values']['show_searchbar'],
    'show_block_searchbar' => $form_state['values']['show_block_searchbar'],
    'placeholder' => $form_state['values']['placeholder'],
    'ac_min_chars' => $form_state['values']['ac_min_chars'],
    'ac_max_suggestions' => $form_state['values']['ac_max_suggestions'],
    'ac_add_matching_label' => $form_state['values']['ac_add_matching_label'],
    'ac_add_context' => $form_state['values']['ac_add_context'],
    'ac_add_facet_name' => $form_state['values']['ac_add_facet_name'],
    'show_facetbox' => $form_state['values']['show_facetbox'],
    'search_type' => $form_state['values']['search_type'],
    // Other settings.
    'use_css_file' => $form_state['values']['use_css_file'],
    'add_rss_functionality' => $form_state['values']['add_rss_functionality'],
    'semantic_connection' => $form_state['values']['semantic_connection'],
  );

  // Update an existing configuration set.
  if (isset($form_state['values']['swid'])) {
    db_update('sonr_webmining_sets')
      ->fields(array(
          'title' => $form_state['values']['title'],
          'config' => serialize($config),
        ))
      ->condition('swid', $form_state['values']['swid'])
      ->execute();

    // Clear the cache for this configuration set.
    $cache_id = 'semantic_connector:sonr_webmining:configuration_set_id:' . $form_state['values']['swid'];
    cache_clear_all($cache_id, 'cache');
  }

  drupal_set_message(t('PoolParty GraphSearch configuration-set %title has been saved.', array('%title' => $previous_configuration_set->getTitle())));
  $form_state['redirect'] = 'admin/config/semantic-drupal/sonr-webmining';
}

/**
 * Add a PP GraphSearch configuration for a predefined PP connection + project.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param int $connection_id
 *   The ID of the PoolParty server connection.
 * @param string $project_id
 *   The ID of the PoolParty project.
 *
 * @return array
 *   The Drupal form array of the confirmation form.
 */
function sonr_webmining_add_config_for_fixed_connection_form($form, &$form_state, $connection_id, $project_id) {
  $connection = SemanticConnector::getConnection('pp_server', $connection_id);
  $pp_config = $connection->getConfig();
  if (!empty($pp_config)) {
    foreach ($pp_config['projects'] as $project) {
      if ($project->id == $project_id) {
        $form['connection_id'] = array(
          '#type' => 'value',
          '#value' => $connection_id,
        );
        $form['project_id'] = array(
          '#type' => 'value',
          '#value' => $project_id,
        );
        $form['description'] = array(
          '#markup' => t('Selected PoolParty server:') . ' <b>' . $connection->getTitle() . '</b><br />' . t('Selected project:') . ' <b>' .$project->title . '</b>',
        );

        if (!empty($pp_config['sonr_configuration']) && isset($pp_config['sonr_configuration']['projects'][$project->id])) {
          if (version_compare($pp_config['sonr_configuration']['version'], '6.1', '>=')) {
            $search_spaces = array();
            foreach ($pp_config['sonr_configuration']['projects'][$project->id]['search_spaces'] as $search_space) {
              $search_spaces[$search_space['id']] = $search_space['name'];
            }
            $form['search_space_id'] = array(
              '#type' => 'select',
              '#title' => t('Select search space'),
              '#options' => $search_spaces,
            );
          }
          else {
            $form['search_space_id'] = array(
              '#type' => 'value',
              '#value' => $project_id,
            );
          }
        }

        // Save and cancel buttons.
        $form['save'] = array(
          '#type' => 'submit',
          '#value' => t('Create configuration'),
          '#prefix' => '<div class="form-actions form-wrapper">',
        );
        $form['cancel'] = array(
          '#type' => 'link',
          '#title' => t('Cancel'),
          '#href' => (isset($_GET['destination']) ? $_GET['destination'] : 'admin/config/semantic-drupal/sonr-webmining'),
          '#suffix' => '</div>',
        );

        return $form;
      }
    }
  }

  $form['description'] = array(
    '#markup' => '<p>' . t('Invalid Semantic Connector connection or project selected.') . '</p>',
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => (isset($_GET['destination']) ? $_GET['destination'] : 'admin/config/semantic-drupal/sonr-webmining'),
  );

  return $form;
}

/**
 * Validation handler for sonr_webmining_add_config_for_fixed_connection_form().
 */
function sonr_webmining_add_config_for_fixed_connection_form_validate($form, &$form_state) {
  if (!isset($form_state['values']['search_space_id']) || empty($form_state['values']['search_space_id'])) {
    form_set_error('search_space_id', t('Please select a search space.'));
  }
}

/**
 * Submit handler for sonr_webmining_add_config_for_fixed_connection_form().
 */
function sonr_webmining_add_config_for_fixed_connection_form_submit($form, &$form_state) {
  $connection = SemanticConnector::getConnection('pp_server', $form_state['values']['connection_id']);
  $server_config = $connection->getConfig();
  $project_id = $form_state['values']['project_id'];
  $search_space_id = $form_state['values']['search_space_id'];

  // The project has a configured PoolParty GraphSearch server.
  if (isset($server_config['sonr_configuration']) && !empty($server_config['sonr_configuration']) && isset($server_config['sonr_configuration']['projects'][$project_id]) && isset($server_config['sonr_configuration']['projects'][$project_id]['search_spaces']) && isset($server_config['sonr_configuration']['projects'][$project_id]['search_spaces'][$search_space_id])) {
    foreach ($server_config['projects'] as $project) {
      if ($project->id == $project_id) {
        $configuration_set_title = 'PoolParty GraphSearch configuration for ' . $connection->getTitle() . ' (' . $project->title . ')';

        // Create a new configuration set.
        $swid = db_insert('sonr_webmining_sets')
          ->fields(array(
            'title' => $configuration_set_title,
            'connection_id' => $connection->getId(),
            'search_space_id' => $search_space_id,
          ))
          ->execute();

        // Add all facets once the set is created.
        $sonr_webmining = new SonrWebmining($swid);
        $config['facets_to_show'] = array_keys($sonr_webmining->getAllFacets());
        db_update('sonr_webmining_sets')
          ->fields(array(
            'config' => serialize(SonrWebminingConfigurationSet::getDefaultConfig()),
          ))
          ->condition('swid', $swid)
          ->execute();

        drupal_set_message(t('PoolParty GraphSearch configuration "%title" has been created.', array('%title' => $configuration_set_title)));
        // Drupal Goto to forward a destination if one is available.
        if (isset($_GET['destination'])) {
          $destination = $_GET['destination'];
          unset($_GET['destination']);
          drupal_goto('admin/config/semantic-drupal/sonr-webmining/' . $swid, array('query' => array('destination' => $destination)));
          exit();
        }
        $form_state['redirect'] = 'admin/config/semantic-drupal/sonr-webmining/' . $swid;
        break;
      }
    }
  }
  // No PoolParty GraphSearch server available for the selected project.
  else {
    drupal_set_message(t('There is no PoolParty GraphSearch configuration available with SearchSpace-ID "%searchspaceid" on PoolParty server "%ppservertitle"', array('%searchspaceid' => $search_space_id,'%ppservertitle' => $connection->getTitle())), 'error');
    $form_state['redirect'] = 'admin/config/semantic-drupal/sonr-webmining';
  }
}

/**
 * The confirmation-form for cloning a sw configuration set.
 */
function sonr_webmining_configuration_set_clone_form($form, &$form_state, $config) {
  $form_state['config'] = $config;
  return confirm_form($form,
    t('Are you sure you want to clone the PoolParty GraphSearch configuration "%title"?', array('%title' => $config->getTitle())),
    'admin/config/semantic-drupal/sonr-webmining',
    '',
    t('Clone configuration'));
}

/**
 * The submit handler confirmation-form for cloning a sw configuration set.
 */
function sonr_webmining_configuration_set_clone_form_submit($form, &$form_state) {
  /** @var \SonrWebminingConfigurationSet $config */
  $config = $form_state['config'];

  $swid = db_insert('sonr_webmining_sets')
    ->fields(array(
      'title' => $config->getTitle() . ' (CLONE)',
      'connection_id' => $config->getConnectionId(),
      'search_space_id' => $config->getSearchSpaceId(),
      'config' => serialize($config->getConfig()),
    ))
    ->execute();

  drupal_set_message(t('%title has been cloned.', array('%title' => $config->getTitle())));
  $form_state['redirect'] = 'admin/config/semantic-drupal/sonr-webmining/' . $swid;
}

/**
 * The confirmation-form for deleting a sw configuration set.
 *
 * @param \SonrWebminingConfigurationSet $config
 *   The configuration to delete.
 *
 * @return array
 *   The confirmation form array.
 */
function sonr_webmining_configuration_set_delete_form($form, &$form_state, $config) {
  $form_state['config'] = $config;
  return confirm_form($form,
    t('Are you sure you want to delete the PoolParty GraphSearch configuration "%title"?', array('%title' => $config->getTitle())),
    'admin/config/semantic-drupal/sonr-webmining',
    t('This action cannot be undone.'),
    t('Delete configuration'));
}

/**
 * The submit handler confirmation-form for deleting a sw configuration set.
 */
function sonr_webmining_configuration_set_delete_form_submit($form, &$form_state) {
  $config = $form_state['config'];

  db_delete('sonr_webmining_sets')
    ->condition('swid', $config->getId())
    ->execute();

  $block_delta_condition = db_or()
    ->condition('delta', 'sonr_webmining_content_' . $config->getId())
    ->condition('delta', 'sonr_webmining_filters_' . $config->getId());

  db_delete('block')
    ->condition($block_delta_condition)
    ->execute();

  drupal_set_message(t('%title has been deleted.', array('%title' => $config->getTitle())));
  $form_state['redirect'] = 'admin/config/semantic-drupal/sonr-webmining';
}

/******************************************************
 * Synchronization functions
 *****************************************************/

/**
 * List all saved PoolParty servers.
 *
 * @return array
 *   The renderable HTML of the list of PoolParty servers.
 */
function sonr_webmining_sync_list() {
  $output = array();

  $output['sonr_webmining_sync_title'] = array(
    '#markup' => '<h3 class="semantic-connector-table-title">' . t('Synchronization of Nodes with a PoolParty GraphSearch server') . '</h3>',
  );

  // Get all the content types related to a PoolParty GraphSearch server.
  $variables = variable_get('sonr_webmining_content_type_push', array());
  $content_types = array();
  if (!empty($variables)) {
    foreach ($variables as $content_type => $settings) {
      if (empty($content_type) || !$settings['active']) {
        continue;
      }
      if (!isset($content_types[$settings['connection_id']])) {
        $content_types[$settings['connection_id']] = array();
      }
      $content_types[$settings['connection_id']][$settings['search_space_id']][] = node_type_get_name($content_type);
    }
  }

  $rows = array();
  $pp_servers = SemanticConnector::getConnectionsByType('pp_server');
  /** @var \SemanticConnectorPPServerConnection $pp_server */
  foreach ($pp_servers as $pp_server) {
    $pp_config = $pp_server->getConfig();
    if (!empty($pp_config['sonr_configuration'])) {
      $title = '<div class="semantic-connector-led" data-server-id="' . $pp_server->getId() . '" data-server-type="pp-server" title="' . t('Checking service') . '"></div>';
      $title .= l($pp_server->getTitle(), $pp_server->getUrl() . '/' . $pp_server->getGraphSearchPath(), array('attributes' => array('target' => '_blank')));

      // Get the search space label.
      $search_space_labels = array();
      $search_spaces = semantic_connector_get_graphsearch_search_spaces($pp_config['sonr_configuration']);
      if (version_compare($pp_config['sonr_configuration']['version'], '6.1', '>=')) {
        foreach ($search_spaces as $search_space) {
          $search_space_labels[$search_space['id']] = (!empty($search_space['name']) ? $search_space['name'] : t('no search space found'));
        }
      }
      else {
        foreach ($pp_config['projects'] as $project) {
          if (isset($pp_config['sonr_configuration']['projects'][$project->id]) && isset($search_spaces[$project->id])) {
            $search_space_labels[$project->id] = $project->title;
            break;
          }
        }
      }

      foreach ($search_space_labels as $search_space_id => $search_space_label) {
        $nodes = (!isset($content_types[$pp_server->getId()]) || !isset($content_types[$pp_server->getId()][$search_space_id])) ? '<div class="semantic-connector-italic">' . t('not yet set') . '</div>' : theme('item_list', array('items' => $content_types[$pp_server->getId()][$search_space_id]));

        $actions = array();
        if (isset($content_types[$pp_server->getId()]) && isset($content_types[$pp_server->getId()][$search_space_id])) {
          $actions = array(
            l(t('Synchronize'), 'admin/config/semantic-drupal/sonr-webmining/synchronization/' . $pp_server->getId() . '/' . $search_space_id . '/update'),
            l(t('Remove'), 'admin/config/semantic-drupal/sonr-webmining/synchronization/' . $pp_server->getId() . '/' . $search_space_id . '/delete'),
          );
        }

        $rows[] = array(
          $title,
          $search_space_label,
          $nodes,
          implode(' | ', $actions),
        );
      }
    }
  }

  $output['sonr_webmining_sync'] = array(
    '#theme' => 'table',
    '#header' => array(
      t('PoolParty GraphSearch server'),
      t('Search space'),
      t('Available in content type'),
      t('Operations'),
    ),
    '#rows' => $rows,
    '#attributes' => array(
      'id' => 'sonr-webmining-sync-table',
      'class' => array('semantic-connector-tablesorter'),
    ),
    '#attached' => array(
      'js' => array(drupal_get_path('module', 'sonr_webmining') . '/js/sonr_webmining.admin.js'),
      'css' => array(drupal_get_path('module', 'sonr_webmining') . '/css/sonr_webmining.admin.css'),
    ),
  );

  // Add the tablesorter library if available.
  $tablesorter_path = 'sites/all/libraries/tablesorter';
  if (module_exists('libraries')) {
    $tablesorter_path = libraries_get_path('tablesorter');
  }
  if (file_exists($tablesorter_path . '/jquery.tablesorter.min.js')) {
    $output['#attached']['js'][] = libraries_get_path('tablesorter') . '/jquery.tablesorter.min.js';
  }

  return $output;
}

/**
 * A form to select the content types for updating or deleting of pings
 * from PoolParty GraphSearch server.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param int $connection_id
 *   A Semantic Connector connection ID.
 * @param string $search_space_id
 *   The ID of the GraphSearch Search Space to use.
 * @param string $operation
 *   The operation type to update or delete the pings.
 *
 * @return array
 *   The Drupal form array.
 */
function sonr_webmining_sync_form($form, &$form_state, $connection_id, $search_space_id, $operation) {
  // Check for the right operations.
  if (!in_array($operation, array('update', 'delete'))) {
    drupal_goto('admin/config/semantic-drupal/sonr-webmining/synchronization');
    exit();
  }

  // Check if the batch is running in background.
  $bid = sonr_webmining_batch_running('admin/config/semantic-drupal/sonr-webmining/synchronization/' . $connection_id . '/' . $search_space_id . '/' . $operation);
  if ($bid !== FALSE) {
    drupal_goto('batch', array('query' => array('op' => 'start', 'id' => $bid)));
    exit();
  }

  $variables = variable_get('sonr_webmining_content_type_push', array());
  $content_types = array();
  if (!empty($variables)) {
    foreach ($variables as $content_type => $settings) {
      if (!empty($content_type) && $connection_id == $settings['connection_id'] && $search_space_id == $settings['search_space_id'] && $settings['active']) {
        $query = new EntityFieldQuery();
        $count = $query->entityCondition('entity_type', 'node')
          ->entityCondition('bundle', $content_type)
          ->count()
          ->execute();
        $option_title = t('!count nodes from content type "@name"', array(
          '!count' => $count,
          '@name' => node_type_get_name($content_type),
        ));
        $content_types[$content_type] = $option_title;
      }
    }
  }

  if (!empty($content_types)) {
    $form['connection_id'] = array(
      '#type' => 'value',
      '#value' => $connection_id,
    );

    $form['search_space_id'] = array(
      '#type' => 'value',
      '#value' => $search_space_id,
    );

    $form['operation'] = array(
      '#type' => 'value',
      '#value' => $operation,
    );

    $form['content_types'] = array(
      '#type' => 'checkboxes',
      '#title' => ($operation == 'update' ? t('Select content types that will be added/updated on the PoolParty GraphSearch server') : t('Select content types that will be removed from the PoolParty GraphSearch server')),
      '#options' => $content_types,
      '#required' => TRUE,
    );

    $form['entities_per_request'] = array(
      '#type' => 'textfield',
      '#title' => t('Nodes per request'),
      '#description' => t('The number of nodes, that get processed during one HTTP request. (Allowed value range: 1 - 100)') . '<br />' . t('The higher this number is, the less HTTP requests have to be sent to the server until the batch finished tagging ALL your nodes, what results in a shorter duration of the bulk tagging process.') . '<br />' . t('Numbers too high can result in a timeout, which will break the whole bulk tagging process.'),
      '#required' => TRUE,
      '#default_value' => 10,
    );

    if (module_exists('background_batch')) {
      global $user;
      $form['inform_user'] = array(
        '#type' => 'checkbox',
        '#title' => 'Send an e-mail out when the process is completed',
      );
      $form['inform_user_email_container'] = array(
        '#type' => 'container',
        '#attributes' => array('style' => array('padding-left:18px;')),
      );
      $form['inform_user_email_container']['inform_user_email'] = array(
        '#type' => 'textfield',
        '#title' => t('E-mail address'),
        '#description' => t('A summary will be sent to this e-mail address when the process is completed.'),
        '#default_value' => $user->mail,
        '#states' => array(
          'visible' => array(':input[name="inform_user"]' => array('checked' => TRUE)),
        ),
      );
    } else {
      $text = t('For more usability with a large number of entities, please install the "<a href="https://www.drupal.org/project/background_process" target="_blank">Background Process</a>" module and activate the "Background Batch" sub-module.') . '<br />';
      $text .= t('This modules takes over the existing Batch API and runs batch jobs in a background process. This means that if you leave the batch page (e.g. you close the browser), the jobs continues, and you can return to the progress indicator later.');
      $form['install_info'] = array(
        '#prefix' => '<div class="form-item">',
        '#suffix' => '</div>',
        '#markup' => $text,
      );
    }

    $form['save'] = array(
      '#type' => 'submit',
      '#value' => ($operation == 'update' ? t('Synchronize') : t('Remove')),
    );

    $form['cancel'] = array(
      '#type' => 'link',
      '#title' => t('Cancel'),
      '#href' => 'admin/config/semantic-drupal/sonr-webmining/synchronization',
      '#suffix' => '</div>',
    );
  }
  else {
    $pp_server = SemanticConnector::getConnection('pp_server', $connection_id);
    drupal_set_message(t('No connected content types found for the PoolParty GraphSearch server "%connection".', array('%connection' => $pp_server->getTitle())), 'error');
    drupal_goto('admin/config/semantic-drupal/sonr-webmining/synchronization');
    exit();
  }

  // Add JavaScript
  drupal_add_js(drupal_get_path('module', 'sonr_webmining') . '/js/sonr_webmining.admin.js');

  return $form;
}

/**
 * Validate-Handler of the form to synchronize nodes with a
 * PoolParty GraphSearch server.
 */
function sonr_webmining_sync_form_validate($form, &$form_state) {
  $values = $form_state['values'];
  $entities_per_request = $values['entities_per_request'];

  if (empty($entities_per_request) || !ctype_digit($entities_per_request) || (int) $entities_per_request == 0 || (int) $entities_per_request > 100) {
    form_set_error('entities_per_request', t('Only values in the range of 1 - 100 are allowed for field "Entities per request"'));
  }

  if (!empty($values['inform_user']) && !empty($values['inform_user_email']) && !valid_email_address($values['inform_user_email'])) {
    form_set_error('inform_user_email', t('Please enter a valid email address.'));
  }
}

function sonr_webmining_sync_form_submit($form, &$form_state) {
  $start_time = time();
  $values = $form_state['values'];

  // Configure the batch data.
  $batch = array(
    'title' => ($values['operation'] == 'update' ? t('Update nodes') : t('Remove nodes')),
    'operations' => array(),
    'init_message' => ($values['operation'] == 'update' ? t('Start with the synchronisation of the nodes.') : t('Start with the removing of the nodes.')),
    'progress_message' => '',
    'finished' => 'sonr_webmining_sync_batch_finished',
  );

  // Set additional data.
  $result = db_select('node', 'n')
    ->fields('n', array('nid'))
    ->condition('n.type', $values['content_types'])
    ->execute();
  $count = $result->rowCount();
  $nids = $result->fetchCol();
  $batch_infos = array(
    'operation' => $values['operation'],
    'total' => $count,
    'start_time' => $start_time,
    'inform_user_email' => (!empty($values['inform_user']) && !empty($values['inform_user_email'])) ? $values['inform_user_email'] : '',
  );

  // Set the synchronization operations.
  for ($i = 0; $i < $count; $i += $values['entities_per_request']) {
    $nodes = array_slice($nids, $i, $values['entities_per_request']);
    $batch['operations'][] = array(
      'sonr_webmining_sync_batch',
      array($nodes, $values['connection_id'], $values['search_space_id'], $batch_infos),
    );
  }

  // Start the batch
  batch_set($batch);
  return TRUE;
}

/**
 * Checks if the background batch is running.
 *
 * @param string $url
 *   The URL of the source.
 *
 * @return int|boolean
 *   The batch ID or FALSE if the batch is not running.
 */
function sonr_webmining_batch_running($url) {
  // Check if the Background Batch module is enabled.
  if (!module_exists('background_batch')) {
    return FALSE;
  }

  // Check if the appropriate batch is running.
  $bids = db_select('batch', 'b')
    ->fields('b', array('bid', 'batch'))
    ->orderBy('b.bid', 'ASC')
    ->execute()
    ->fetchAllKeyed(0, 1);

  foreach ($bids as $bid => $data) {
    $batch = unserialize($data);
    if ($batch['source_url'] == $url) {
      return $bid;
      break;
    }
  }

  return FALSE;
}

/******************************************************
 * Agents functions
 *****************************************************/

/**
 * List all saved PoolParty GraphSearch agents.
 *
 * @return array
 *   The rendered HTML of the list of agents.
 */
function sonr_webmining_agent_list() {
  $output = array();
  $connections = SemanticConnector::getConnectionsByType('pp_server');
  if (!empty($connections)) {
    $intervals = _sonr_webmining_get_intervals();
    /** @var \SemanticConnectorPPServerConnection $connection */
    foreach ($connections as $connection) {
      $connection_config = $connection->getConfig();
      if (!empty($connection_config['sonr_configuration'])) {
        /** @var \SemanticConnectorSonrApi $sonr_api */
        $sonr_api = $connection->getApi('sonr');
        $connection_id = $connection->getId();

        $sonr_config = $connection_config['sonr_configuration'];
        if (isset($sonr_config['version']) && version_compare($sonr_config['version'], '6.1', '>=')) {
          $search_spaces = semantic_connector_get_graphsearch_search_spaces($sonr_config);
        }
        // Fallback for older versions and errors.
        if (!isset($search_spaces) || empty($search_spaces)) {
          $search_spaces = ['' => ['name' => '']];
        }

        foreach ($search_spaces as $search_space_id => $search_space) {
          $link = l($connection->getTitle(), 'admin/config/semantic-drupal/semantic-connector', array('fragment' => 'pp-server-' . $connection_id));
          $title = '<h3 class="semantic-connector-table-title">' . t('Agents for PoolParty GraphSearch on PoolParty server "!link"', array('!link' => $link)) . (!empty($search_space_id) ? ' ' . t('for agent %agentname', array('%agentname' => $search_space['name'])) : '') . '</h3>';
          $output['agents_title_' . $connection_id . '_' . $search_space_id] = array(
            '#markup' => $title,
          );

          $rows = array();
          $indexed_agents = array();
          if (($agents = $sonr_api->getAgents($search_space_id)) !== FALSE) {
            $indexed_agents = $sonr_api->getIndexedAgents($search_space_id);
            foreach ($agents as $agent) {
              $agent_id_full = implode('|', array($connection_id, $search_space_id, urlencode($agent->id)));
              $actions = array(
                l(t('Edit'), 'admin/config/semantic-drupal/sonr-webmining/agents/' . $agent_id_full . '/edit'),
                l(t('Delete'), 'admin/config/semantic-drupal/sonr-webmining/agents/' . $agent_id_full . '/delete'),
                l(t('Start crawling'), 'admin/config/semantic-drupal/sonr-webmining/agents/' . $agent_id_full . '/run'),
              );
              $rows[] = array(
                $agent->configuration['source'],
                l($agent->configuration['url'], $agent->configuration['url'], array('attributes' => array('target' => '_blank'))),
                $intervals[$agent->configuration['periodMillis']],
                _sonr_webmining_percentage_bar($agent),
                implode(' | ', $actions),
              );
              if (isset($indexed_agents[$agent->configuration['source']])) {
                unset($indexed_agents[$agent->configuration['source']]);
              }
            }
          }
          $output['agents_' . $connection_id . '_' . $search_space_id] = array(
            '#theme' => 'table',
            '#header' => array(
              t('Agent name'),
              t('URL'),
              t('Crawling interval'),
              t('Next crawling'),
              t('Operations'),
            ),
            '#rows' => $rows,
          );

          if (!empty($indexed_agents)) {
            $table = array();
            $rows = array();
            foreach ($indexed_agents as $agent_id) {
              $actions = array(
                l(t('Delete indexed feed items'), 'admin/config/semantic-drupal/sonr-webmining/agents/' . $connection_id . '/' . $search_space_id . '/' . urlencode($agent_id) . '/delete'),
              );
              $rows[] = array(
                $agent_id,
                implode(' | ', $actions),
              );
            }
            $table['deleted_agents'] = array(
              '#theme' => 'table',
              '#header' => array(
                t('Source name'),
                t('Operations'),
              ),
              '#rows' => $rows,
            );
            $output['deleted_agents_fieldset_' . $connection_id . '_' . $search_space_id] = array(
              '#theme' => 'fieldset',
              '#title' => t('Agents that have been deleted, but their feed items are still stored in the search index'),
              '#collapsible' => TRUE,
              '#collapsed' => TRUE,
              '#children' => drupal_render($table),
              '#attributes' => array(
                'class' => array('collapsible', 'collapsed', 'deleted_agents'),
              ),
              '#attached' => array(
                'js' => array('misc/collapse.js', 'misc/form.js'),
              ),
            );
          }
        }
      }
    }
  }
  $output['#attached'] = array('css' => array(drupal_get_path('module', 'sonr_webmining') . '/css/sonr_webmining.admin.css'));

  return $output;
}

/**
 * The form to add or edit a PoolParty GraphSearch agents.
 *
 * @param array $config
 *   Array of saved configuration.
 *
 * @return array
 *   The Drupal form array.
 */
function sonr_webmining_agent_form($form, &$form_state, $config = NULL) {
  $form['source'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('Name of the agent.'),
    '#size' => 35,
    '#maxlength' => 60,
    '#default_value' => (!is_null($config)) ? $config['source'] : '',
    '#required' => TRUE,
  );

  $form['url'] = array(
    '#type' => 'textfield',
    '#title' => t('RSS URL'),
    '#description' => t('RSS URL of the source.'),
    '#size' => 35,
    '#maxlength' => 255,
    '#default_value' => (!is_null($config)) ? $config['url'] : '',
    '#required' => TRUE,
  );

  /* The space key is not important for now
  $form['spaceKey'] = array(
    '#type' => 'textfield',
    '#title' => t('Type'),
    '#description' => t('The type of the source (e.g. internal, external) shown in a separate facet.'),
    '#size' => 35,
    '#maxlength' => 60,
    '#default_value' => (!is_null($config)) ? $config['spaceKey'] : '',
  );
  */

  $form['periodMillis'] = array(
    '#type' => 'select',
    '#title' => t('Crawling period'),
    '#description' => t('The period when the agent is to be re-crawled.'),
    '#options' => _sonr_webmining_get_intervals(),
    '#default_value' => (!is_null($config)) ? $config['periodMillis'] : '86400000',
    '#required' => TRUE,
  );

  // Update existing config.
  if (!is_null($config)) {
    $form['connection_id'] = array(
      '#type' => 'hidden',
      '#value' => $config['connection_id'],
    );
    $form['search_space_id'] = array(
      '#type' => 'hidden',
      '#value' => $config['search_space_id'],
    );
    $form['agentid'] = array(
      '#type' => 'hidden',
      '#value' => $config['id'],
    );
  }
  // Create a new config --> choose sOnr-configuration.
  else {
    $sonr_options = array();
    $search_space_options = array();
    $connections = SemanticConnector::getConnectionsByType('pp_server');
    if (!empty($connections)) {
      /** @var \SemanticConnectorPPServerConnection $connection */
      foreach ($connections as $connection) {
        $connection_config = $connection->getConfig();
        if (!empty($connection_config['sonr_configuration'])) {
          $sonr_options[$connection->getId()] = $connection->getTitle();

          $sonr_config = $connection_config['sonr_configuration'];
          if (isset($sonr_config['version']) && version_compare($sonr_config['version'], '6.1', '>=')) {
            $search_space_options[$connection->getId()] = [];
            $search_spaces = semantic_connector_get_graphsearch_search_spaces($sonr_config);
            foreach ($search_spaces as $search_space) {
              $search_space_options[$connection->getId()][$search_space['id']] = $search_space['name'];
            }
          }
        }
      }
    }
    $form['connection_id'] = array(
      '#type' => 'select',
      '#title' => t('PoolParty GraphSearch server'),
      '#description' => t('Choose the PoolParty GraphSearch server this agent will be created for.'),
      '#options' => $sonr_options,
      '#required' => TRUE,
    );

    foreach ($search_space_options as $connection_id => $search_space_connection_options) {
      $form['search_space_id_' . $connection_id] = array(
        '#type' => 'select',
        '#title' => t('Search space'),
        '#description' => t('Select the search space this agent will be created for.'),
        '#options' => $search_space_connection_options,
        '#states' => array(
          'visible' => array(':input[name="connection_id"]' => array('value' => $connection_id)),
          'required' => array(':input[name="connection_id"]' => array('value' => $connection_id)),
        ),
      );
    }
  }

  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/semantic-drupal/sonr-webmining/agents',
  );

  return $form;
}

/**
 * The validation handler for the PoolParty GraphSearch agent admin form.
 */
function sonr_webmining_agent_form_validate($form, &$form_state) {
  if (!valid_url($form_state['values']['url'], TRUE)) {
    form_set_error('url', t('The field URL must be a valid URL.'));
  }

  if (!is_null($form_state['values']['connection_id']) && isset($form['search_space_id_' . $form_state['values']['connection_id']])) {
    $form_item_id = 'search_space_id_' . $form_state['values']['connection_id'];
    if (!isset($form_state['values'][$form_item_id]) || empty($form_state['values'][$form_item_id])) {
      form_set_error($form_item_id, t('A search space has to be selected for the selected GraphSearch server.'));
    }
  }
}

/**
 * The submit handler for the PoolParty GraphSearch agent admin form.
 */
function sonr_webmining_agent_form_submit($form, &$form_state) {
  global $user;

  /** @var \SemanticConnectorSonrApi $sonr_api */
  $sonr_api = SemanticConnector::getConnection('pp_server', $form_state['values']['connection_id'])->getApi('sonr');
  $config = array(
    'username' => $user->name,
    'source' => $form_state['values']['source'],
    'url' => $form_state['values']['url'],
    'periodMillis' => $form_state['values']['periodMillis'],
  );

  if (isset($form_state['values']['agentid'])) {
    $response = $sonr_api->updateAgent($form_state['values']['agentid'], $config, $form_state['values']['search_space_id']);
  }
  else {
    $search_space_id = '';
    if (isset($form_state['values']['search_space_id_' . $form_state['values']['connection_id']])) {
      $search_space_id = $form_state['values']['search_space_id_' . $form_state['values']['connection_id']];
    }
    $response = $sonr_api->addAgent($config, $search_space_id);
  }
  if ($response) {
    drupal_set_message(t('PoolParty GraphSearch agent %title has been saved.', array('%title' => $form_state['values']['source'])));
  }
  else {
    drupal_set_message(t('PoolParty GraphSearch agent %title has not been saved.', array('%title' => $form_state['values']['source'])), 'error');
  }
  $form_state['redirect'] = 'admin/config/semantic-drupal/sonr-webmining/agents';
}

/**
 * The confirmation-form for deleting a PoolParty GraphSearch agent.
 */
function sonr_webmining_agent_delete_form($form, &$form_state, $config) {
  $form_state['config'] = $config;
  $form['question'] = array(
    '#markup' => '<p>' . t('Are you sure you want to delete the agent "%source"?', array('%source' => $config['source'])) . '<br />' .
    t('This action cannot be undone.') . '</p>',
  );
  $form['deleteIndex'] = array(
    '#type' => 'checkbox',
    '#default_value' => true,
    '#title' => t('Delete all indexed feed items also.'),
  );
  $form['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete agent'),
  );

  return $form;
}

/**
 * The submit handler for deleting a PoolParty GraphSearch agent.
 */
function sonr_webmining_agent_delete_form_submit($form, &$form_state) {
  $config = $form_state['config'];
  /** @var \SemanticConnectorSonrApi $sonr_api */
  $sonr_api = SemanticConnector::getConnection('pp_server', $config['connection_id'])->getApi('sonr');
  $response = $sonr_api->deleteAgent($config['id'], $config['search_space_id']);

  if ($form_state['values']['deleteIndex']) {
    $sonr_api->deleteIndex($config['source'], $config['search_space_id']);
  }

  if ($response) {
    drupal_set_message(t('%source has been deleted.', array('%source' => $config['source'])));
  }
  else {
    drupal_set_message(t('%source has not been deleted.', array('%source' => $config['source'])), 'error');
  }
  $form_state['redirect'] = 'admin/config/semantic-drupal/sonr-webmining/agents';
}

/**
 * The confirmation-form for running a PoolParty GraphSearch agent manually.
 */
function sonr_webmining_agent_run_form($form, &$form_state, $config) {
  $form_state['config'] = $config;
  $form['question'] = array(
    '#markup' => '<p>' . t('Are you sure you want to start crawling the RSS feed for the agent "%source"?', array('%source' => $config['source'])) . '</p>',
  );
  $form['run'] = array(
    '#type' => 'submit',
    '#value' => t('Start crawling'),
  );

  return $form;
}

/**
 * The submit handler for running a PoolParty GraphSearch agent manually.
 */
function sonr_webmining_agent_run_form_submit($form, &$form_state) {
  $config = $form_state['config'];
  /** @var \SemanticConnectorSonrApi $sonr_api */
  $sonr_api = SemanticConnector::getConnection('pp_server', $config['connection_id'])->getApi('sonr');
  $response = $sonr_api->runAgent($config['id'], $config['search_space_id']);

  if ($response) {
    drupal_set_message(t('%source has been started.', array('%source' => $config['source'])));
  }
  else {
    drupal_set_message(t('%source has not been started.', array('%source' => $config['source'])), 'error');
  }
  $form_state['redirect'] = 'admin/config/semantic-drupal/sonr-webmining/agents';
}

/**
 * The confirmation-form for deleting the index of a PoolParty GraphSearch agent.
 */
function sonr_webmining_agent_index_delete_form($form, &$form_state, $connection_id, $search_space_id, $agent_id) {
  $form_state['connection_id'] = $connection_id;
  $form_state['search_space_id'] = $search_space_id;
  $form_state['agent_id'] = $agent_id;
  $form['question'] = array(
    '#markup' => '<p>' . t('Are you sure you want to delete all indexed feed items for the agent "%agent"?', array('%agent' => $agent_id)) . '<br />' .
      t('This action cannot be undone.') . '</p>',
  );
  $form['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete indexed feed times'),
  );

  return $form;
}

/**
 * The submit handler for deleting the index of a PoolParty GraphSearch agent.
 */
function sonr_webmining_agent_index_delete_form_submit($form, &$form_state) {
  /** @var \SemanticConnectorSonrApi $sonr_api */
  $sonr_api = SemanticConnector::getConnection('pp_server', $form_state['connection_id'])->getApi('sonr');
  $response = $sonr_api->deleteIndex($form_state['agent_id'], $form_state['search_space_id']);

  if ($response) {
    drupal_set_message(t('Indexed feed items have been deleted for agent %agent.', array('%agent' => $form_state['agent_id'])));
  }
  else {
    drupal_set_message(t('Indexed feed items have not been deleted for agent %agent.', array('%agent' => $form_state['agent_id'])), 'error');
  }
  $form_state['redirect'] = 'admin/config/semantic-drupal/sonr-webmining/agents';
}

/**
 * Returns different time intervals.
 *
 * @return array
 *  Array with different time intervals.
 */
function _sonr_webmining_get_intervals() {
  return array(
    '3600000' => t('hourly'),
    '86400000' => t('daily'),
    '604800000' => t('weekly'),
  );
}

/******************************************************
 * Helper functions
 *****************************************************/

/**
 * Shows a bar in percentage when the next run starts.
 * TODO: If an Agent has problems then send an email (adapt PoolParty GraphSearch server configuration)
 *
 * @param object $agent
 *   An agent
 *
 * @return string
 *   The bar
 */
function _sonr_webmining_percentage_bar($agent) {
  if ($agent->status['running']) {
    return '<div class="percentage-bar running"><span class="bar" style="width: 100%"></span>
      <span class="tooltip">
        <strong>' . t('Agent is crawled') . '</strong>
      </span></div>
    ';
  }

  $period = $agent->configuration['periodMillis'] / 1000;
  $time_to_next_run = $agent->status['nextRun'] / 1000 - time();
  $time_to_last_run = time() - $agent->status['lastRun'] / 1000;

  if ($time_to_next_run < 0) {
    $last_run = _sonr_webmining_time_to_text($time_to_last_run);
    return '<div class="percentage-bar error"><span class="bar" style="width: 100%"></span>
      <span class="tooltip">
        <strong>' . t('Last crawling') . ':</strong><br />' . $last_run . ' ago<br />
        <strong>' . t('Agent problem') . ':</strong><br />' . t('Please start agent manually!') . '
      </span></div>
    ';
  }

  $percentage = round(100 * $time_to_last_run / $period);

  $next_run = _sonr_webmining_time_to_text($time_to_next_run, 'ceil');
  $last_run = _sonr_webmining_time_to_text($time_to_last_run, 'floor');

  return '<div class="percentage-bar"><span class="bar" style="width: ' . $percentage . '%"></span>
    <span class="tooltip">
      <strong>' . t('Next crawling') . ':</strong><br /> ' . t('in @time', array('@time' => $next_run)) . '<br />
      <strong>' . t('Last crawling') . ':</strong><br /> ' . t('@time ago', array('@time' => $last_run)) . '
    </span></div>
  ';
}

/**
 * Converts a timestamp into human readable format
 *
 * @param int $time
 *   A timestamp
 * @param string $round
 *   Type of rounding (round|ceil|floor)
 *
 * @return string
 *   A human readable time
 */
function _sonr_webmining_time_to_text($time, $round='round') {
  $time = $time / 60;
  $minutes = (int) $time;
  if ($minutes < 60) {
    return t('!minutes min', array('!minutes' => $minutes));
  }

  $time = $time / 60;
  $hours = (int) $time;
  $minutes = $round(($time - $hours) * 60);
  if ($minutes >= 60) {
    $hours++;
    $minutes = 0;
  }
  if ($hours < 24) {
    return t('!hours hr !minutes min', array('!hours' => $hours, '!minutes' => $minutes));
  }

  $time = $time / 24;
  $days = (int) $time;
  $hours = $round(($time - $days) * 24);
  if ($hours >= 24) {
    $days++;
    $hours = 0;
  }
  return t('!days days !hours hours', array('!days' => $days, '!hours' => $hours));
}
