<?php

/**
 * @file
 * The PoolParty GraphSearch class for searching on the frontend.
 */

class SonrWebmining {

  protected $id;
  protected $search_space_id;
  protected $filters;
  protected $title;
  /** @var \SemanticConnectorPPServerConnection */
  protected $connection;
  /** @var \SemanticConnectorSonrApi */
  protected $sonrApi;
  protected $result;
  protected $config;

  /**
   * Constructor of the SonrWebmining-class.
   *
   * @param int $config_id
   *   ID of the configuration set
   */
  public function __construct($config_id) {
    // Load configuration set.
    $config = sonr_webmining_config_load($config_id);
    if (($this->id = $config->getId()) > 0) {
      $this->search_space_id = $config->getSearchSpaceId();
      $this->filters = array();
      $this->title = $config->getTitle();
      $this->connection = $config->getConnection();
      $this->sonrApi = $config->getConnection()->getApi('sonr');
      $this->result = NULL;
      $this->config = $config->getConfig();
    }
  }

  /**
   * This method checks if the PP GraphSearch server exists and is running.
   *
   * @return bool
   *   TRUE if the service is available, FALSE if not
   */
  public function availableApi() {
    return $this->sonrApi->available();
  }

  /**
   * Getter-function for the id-variable.
   *
   * @return int
   *   ID of the configuration set
   */
  public function getId() {
    return $this->id;
  }

  /**
   * Getter-function for the search_space_id-variable.
   *
   * @return string
   *   ID of the used PoolParty project.
   */
  public function getSearchSpaceId() {
    return $this->search_space_id;
  }

  /**
   * Setter-function for the filters-variable.
   *
   * @param array $filters
   *   Array of filters
   */
  public function setFilters($filters) {
    if (is_array($filters) && !empty($filters) && !is_null($filters)) {
      foreach ($filters as $key => $filter) {
        if (is_array($filter)) {
          $filter = (object) $filter;
        }
        $filters[$key] = $filter;
      }
      $this->filters = $filters;
    }
  }

  /**
   * Getter-function for the filters-variable.
   *
   * @return array
   *   Array of filters
   */
  public function getFilters() {
    return $this->filters;
  }

  /**
   * Getter-function for the result-variable.
   *
   * @return array
   *   Array of results
   */
  public function getResult() {
    return $this->result;
  }

  /**
   * Getter-function for the config-variable.
   *
   * @return array
   *   Config of the configuration set
   */
  public function getConfig() {
    return $this->config;
  }

  /**
   * Returns the URL parameters (uri and search) as a list of filter objects.
   *
   * @return array
   *   A list of filter objects.
   */
  public function getFiltersFromUrlParameter() {
    $filters = array();
    // Get all uri parameters if it is set.
    if (isset($_GET['uri'])) {
      if (!is_array($_GET['uri'])) {
        $_GET['uri'] = array($_GET['uri']);
      }
      // Get all aggregated facets.
      $facet_modes = array();
      foreach ($this->config['facets_to_show'] as $facet) {
        $facet_modes[$facet['facet_id']] = $facet['facet_mode'];
      }
      // Create the mapping between conceptScheme URI and facet name.
      $facet_map = array();
      if ($field_config = $this->sonrApi->getFieldConfig($this->getSearchSpaceId())) {
        foreach ($field_config['searchFields'] as $config) {
          if ($config['facet'] && isset($config['conceptScheme'])) {
            $facet_map[$config['conceptScheme']] = $config['name'];
          }
        }
      }

      foreach ($_GET['uri'] as $uri) {
        if (!valid_url($uri)) {
          continue;
        }

        // Get the concept.
        $concept = $this->getConcept($uri, array('skos:prefLabel', 'skos:ConceptScheme', 'skos:broader'));
        if (!empty($concept)) {
          $field = SemanticConnectorSonrApi::ATTR_ALL_CONCEPTS;
          $concept_schemes = $this->sortConceptUris($concept->conceptSchemes, $facet_modes, $facet_map);
          foreach ($concept_schemes as $concept_scheme) {
            $facet = isset($facet_map[$concept_scheme]) ? $facet_map[$concept_scheme] : '';
            // Check if the facet is shown, the facet mode is aggregated, the
            // concept is a topConcept and the facet is aggregated
            // then show the result with aggregated concepts.
            if ($facet && isset($facet_modes[$facet]) && $facet_modes[$facet] == 'aggregated' && !isset($concept->broaders)) {
              $field = $facet;
              break;
            }
            // Check if the facet is shown, the facet mode is tree and the facet
            // is aggregated then show the result with aggregated concepts.
            elseif ($facet && isset($facet_modes[$facet]) && ($facet_modes[$facet] == 'tree' || $facet_modes[$facet] == 'list')) {
              $field = $facet;
              break;
            }
          }

          $filters[] = (object) array(
            'field' => $field,
            'value' => $uri,
            'label' => $concept->prefLabel,
          );
        }
      }
    }

    // Get all search parameters if it is set and allowed.
    if (isset($_GET['search']) && strpos($this->config['search_type'], 'free-term') !== FALSE) {
      $phrases = is_array($_GET['search']) ? $_GET['search'] : explode(' ', $_GET['search']);
      foreach ($phrases as $phrase) {
        $filters[] = (object) array(
          'field' => SemanticConnectorSonrApi::ATTR_CONTENT,
          'value' => strip_tags(trim($phrase)),
        );
      }
    }

    return array_unique($filters, SORT_REGULAR);
  }

  /**
   * Search for aggregated content.
   *
   * @param int $page
   *   (optional) The page to start searching at
   *
   * @return bool
   *   TRUE if the search was successful, FALSE if not
   */
  public function search($page = 0) {
    $facets = array();
    if (!empty($this->config['facets_to_show'])) {
      foreach ($this->config['facets_to_show'] as $facet) {
        $facets[$facet['facet_id']] = $facet;
      }
    }
    $parameters = array(
      'count' => $this->config['items_per_page'],
      'start' => $page * $this->config['items_per_page'],
      'maxFacetCount' => $this->config['facet_max_items'],
    );

    // Add additional data for every result item
    if ($this->config['show_sentiment']) {
      $this->sonrApi->addCustomAttribute(SemanticConnectorSonrApi::ATTR_SENTIMENT);
    }
    if ($this->config['show_tags']) {
      $this->sonrApi->addCustomAttribute(SemanticConnectorSonrApi::ATTR_ALL_CONCEPTS);
    }
    $this->sonrApi->addCustomAttribute(SemanticConnectorSonrApi::ATTR_CONTENT_TYPE);
    $variables = variable_get('sonr_webmining_content_type_push', array());
    $taxonomies = sonr_webmining_get_all_connected_taxonomies($variables, 'machine_name');
    foreach ($taxonomies as $taxonomy) {
      $this->sonrApi->addCustomAttribute('dyn_lit_' . $taxonomy);
    }

    $custom_attributes = $this->sonrApi->getCustomAttributes();
    drupal_alter('sonr_webmining_custom_attributes', $custom_attributes, $this);
    $this->sonrApi->setCustomAttributes($custom_attributes);

    // Call the search method.
    $this->result = $this->sonrApi->search($this->getSearchSpaceId(), $facets, $this->filters, $parameters);
    // var_dump($this->result['total']); exit;

    // Prepare the facets.
    if (is_array($this->result) && !empty($this->result['facetList'])) {
      foreach ($this->result['facetList'] as $key => $facet) {
        if ($facets[$facet['field']]['selected']) {
          $facet['facets'] = array_slice($facet['facets'], 0, $facets[$facet['field']]['max_items']);
          if ($facets[$facet['field']]['facet_mode'] == 'tree') {
            $facet['facets'] = $this->clipTree($facet['facets'], $facets[$facet['field']]['max_items'], $facets[$facet['field']]['tree_depth']);
          }
          if (!empty($facets[$facet['field']]['name'])) {
            $facet['label'] = $facets[$facet['field']]['name'];
          }
          $this->result['facetList'][$facet['field']] = $facet;
        }
        unset($this->result['facetList'][$key]);
      }

      // Sort the facets.
      $this->result['facetList'] = array_replace(array_intersect_key($facets, $this->result['facetList']), $this->result['facetList']);
    }

    return (!is_null($this->result) && $this->result !== FALSE);
  }

  /**
   * Get all available facets of the connected PoolParty GraphSearch-instance.
   *
   * @return array
   *   An associative array of facets: facet_id => facet_label
   */
  public function getAllFacets() {
    return $this->sonrApi->getFacets($this->getSearchSpaceId());
  }

  /**
   * Search for similar documents.
   *
   * @param string $uri
   *   URI of a document
   *
   * @return bool
   *   TRUE if the search was successful, FALSE if not
   */
  public function searchSimilars($uri) {
    $parameters = array(
      'count' => $this->config['similar_max_items'],
    );

    // Add additional data for every result item
    if ($this->config['show_sentiment']) {
      $this->sonrApi->addCustomAttribute(SemanticConnectorSonrApi::ATTR_SENTIMENT);
    }
    if ($this->config['show_tags']) {
      $this->sonrApi->addCustomAttribute(SemanticConnectorSonrApi::ATTR_ALL_CONCEPTS);
    }

    $this->result = $this->sonrApi->getSimilar($uri, $this->getSearchSpaceId(), $parameters);

    return (!is_null($this->result) && $this->result !== FALSE);
  }

  /**
   * Returns a concept with the URI $uri.
   *
   * @param string $uri
   *   The URI of a concept.
   * @param array $properties
   *   Array of additional concept properties that will be fetched.
   *   If empty all data of the concept will be fetched.
   * @param string $language
   *   The preferred language for the prefLabel.
   *
   * @return object
   *   The concept as an object.
   */
  public function getConcept($uri, array $properties = array(), $language = '') {
    if (!valid_url($uri)) {
      return NULL;
    }
    // Open a new connection to the PPT.
    $ppt_connection = SemanticConnector::getConnection('pp_server');
    $ppt_connection->setUrl($this->connection->getUrl());
    $ppt_connection->setCredentials($this->connection->getCredentials());
    // Get the API of the PPT.
    $pptApi = $ppt_connection->getApi('PPT');
    $config = $this->connection->getConfig();
    // Get the concept.
    $properties = empty($properties) ? array('all') : $properties;
    foreach (array_keys($config['sonr_configuration']['projects']) as $project_id) {
      $concept = $pptApi->getConcept($project_id, $uri, $properties, $language);
      if (!empty($concept)) {
        break;
      }
    }

    if (empty($concept)) {
      return NULL;
    }

    return $concept;
  }

  /**
   * Returns the prefLabel of a concept with the URI $uri.
   *
   * @param string $uri
   *   The URI of a concept.
   * @param string $language
   *   The preferred language for the prefLabel.
   *
   * @return string
   *   The prefLabel of the concept.
   */
  public function getLabelOfConcept($uri, $language = '') {
    $concept = $this->getConcept($uri, array('skos:prefLabel'), $language);
    return empty($concept) ? '' : $concept->prefLabel;
  }

  /**
   * Show aggregated content.
   *
   * @param string $display_type
   *   The way of displaying the data, can be "html" or "rss"
   * @param array $display_parameters
   *   Array of parameters depending on the chosen display type:
   *     for html:
   *       - bool add_filter_form:
   *          TRUE if a filter form shell be added,
   *          FALSE if not
   *     for rss:
   *       no parameters yet
   *
   * @return string
   *   The rendered content
   */
  public function themeContent($display_type = 'html', $display_parameters = array()) {
    // Search was not yet started, or didn't succeed.
    if (is_null($this->result)) {
      drupal_set_message(t('There was an error trying to get the search results from PoolParty GraphSearch.'), 'error');
    }
    else if ($this->result === FALSE) {
      drupal_set_message(t('The PoolParty GraphSearch server is currently not available. Please try again later or contact the server administrator.'), 'warning');
    }

    switch ($display_type) {
      case 'html':
        return $this->themeAsView($display_parameters);

      case 'rss':
        return $this->themeAsRSSfeed($display_parameters);
    }
  }

  /**
   * Show the filter-form of the aggregated content.
   *
   * @return string
   *   The rendered content
   */
  public function themeFilters() {
    // No filters should be shown or the search was not yet started
    // (or didn't succeed).
    if (is_null($this->result)) {
      return '';
    }

    drupal_add_js(array('sonr_webmining' => array('hide_empty_facet' => $this->config['hide_empty_facet'])), 'setting');

    $array_to_render = drupal_get_form('sonr_webmining_filter_form', $this);
    return drupal_render($array_to_render);
  }

  /**
   * Show the search bar with the textfield and buttons.
   *
   * @return string
   *   The rendered content
   */
  protected function themeSearchBar() {
    if (!isset($this->config['show_searchbar']) || !$this->config['show_searchbar']) {
      drupal_add_js(array('sonr_webmining' => array(
        'search_bar' => FALSE,
        'search_type' => $this->config['search_type'],
      )), 'setting');
      return '';
    }
    drupal_add_library('system', 'ui.autocomplete');
    drupal_add_js(array('sonr_webmining' => array(
      'search_bar' => TRUE,
      'min_chars' => $this->config['ac_min_chars'],
      'max_suggestions' => $this->config['ac_max_suggestions'],
      'add_matching_label' => $this->config['ac_add_matching_label'],
      'add_context' => $this->config['ac_add_context'],
      'search_type' => $this->config['search_type'],
    )), 'setting');

    $array_to_render = drupal_get_form('sonr_webmining_search_bar_form', $this);
    return drupal_render($array_to_render);
  }

  /**
   * Show the facet box for the selected concepts and free terms.
   *
   * @return string
   *   The rendered content
   */
  protected function themeFacetBox() {
    $array_to_render = drupal_get_form('sonr_webmining_facet_box_form', $this);
    return drupal_render($array_to_render);
  }

  /**
   * Show the "more like this" link for similar documents.
   *
   * @return string
   *   The rendered content
   */
  public function themeSimilars() {
    $display_parameters = array(
      'only_documents' => TRUE,
    );
    return $this->themeAsView($display_parameters);
  }

  /**
   * Show the search bar in a block with the textfield.
   *
   * @return string
   *   The rendered content
   */
  public function themeBlockSearchBar() {
    if (!isset($this->config['show_block_searchbar']) || !$this->config['show_block_searchbar']) {
      return '';
    }
    $path = sonr_webmining_get_page_path($this->getId());
    drupal_add_library('system', 'ui.autocomplete');
    drupal_add_js(drupal_get_path('module', 'sonr_webmining') . '/js/sonr_webmining_block_search_bar.js');

    $array_to_render = drupal_get_form('sonr_webmining_block_search_bar_form', $this);
    return drupal_render($array_to_render);
  }

  /**
   * Theme PoolParty GraphSearch results as a view.
   *
   * @param array $variables
   *   Array of variables
   *
   * @return string
   *   The rendered content
   */
  protected function themeAsView($variables = array()) {
    global $base_url;

    $view = new View();
    if (!isset($variables['only_documents']) || !$variables['only_documents']) {
      $view->name = 'sonr_webmining_list';
    }
    else {
      $view->name = 'sonr_webmining_list_short';
    }
    $view->description = 'Show a PoolParty GraphSearch list.';
    $view->human_name = 'PoolParty GraphSearch List';
    $handler = $view->new_display('default', 'Master', 'default');
    $header_content = '';

    // Add search bar.
    if (!isset($variables['only_documents']) || !$variables['only_documents']) {
      $header_content .= $this->themeSearchBar();

      // Add the facet box for the selected concepts (facets).
      if ($this->config['show_facetbox']) {
        $header_content .= $this->themeFacetBox();
      }

      // Add an RSS-button in the header of the view if required.
      if ($this->config['add_rss_functionality']) {
        $rss_link = $base_url . '/sonr-webmining/get-rss/' . $this->id . ((!empty($this->filters)) ? '?' . http_build_query(array('filters' => $this->filters)) : '');
        $header_content .= l(t('RSS Feed'), $rss_link, array('attributes' => array('class' => array('rss-link'), 'target' => '_blank')));
      }

      // Add a total count of results.
      if ($this->config['show_results_count']) {
        $header_content .= '<div class="sonr-webmining-results-count">' . $this->result['total'] . ' ' . t('results') . '</div>';
      }
    }

    // Header: Global: Text area.
    if (!empty($header_content)) {
      $handler->display->display_options['header']['area']['id'] = 'area';
      $handler->display->display_options['header']['area']['table'] = 'views';
      $handler->display->display_options['header']['area']['field'] = 'area';
      $handler->display->display_options['header']['area']['content'] = $header_content;
      $handler->display->display_options['header']['area']['format'] = 'full_html';
    }

    // Add additional styling.
    $classes_array = array();
    if (!$this->config['separate_blocks']) {
      $classes_array[] = 'sonr-webmining-area-right';
    }
    else {
      $classes_array[] = 'sonr-webmining-area-list';
    }

    if (!$this->config['first_page_only']) {
      $classes_array[] = 'views-load-more';
    }

    if (!empty($classes_array)) {
      $handler->display->display_options['css_class'] = implode(' ', $classes_array);
    }

    // Field: Content: Path = ID.
    $handler->display->display_options['fields']['sonr_webmining_path']['id'] = 'nid';
    $handler->display->display_options['fields']['sonr_webmining_path']['table'] = 'node';
    $handler->display->display_options['fields']['sonr_webmining_path']['field'] = 'nid';
    $handler->display->display_options['fields']['sonr_webmining_path']['exclude'] = TRUE;

    /* Field: Content: Type */
    $handler->display->display_options['fields']['sonr_webmining_source']['id'] = 'type';
    $handler->display->display_options['fields']['sonr_webmining_source']['table'] = 'node';
    $handler->display->display_options['fields']['sonr_webmining_source']['field'] = 'type';
    $handler->display->display_options['fields']['sonr_webmining_source']['label'] = '';
    $handler->display->display_options['fields']['sonr_webmining_source']['exclude'] = TRUE;
    $handler->display->display_options['fields']['sonr_webmining_source']['element_label_colon'] = FALSE;
    $handler->display->display_options['fields']['sonr_webmining_source']['machine_name'] = TRUE;

    /* Field: Content: Post date */
    $handler->display->display_options['fields']['created']['id'] = 'created';
    $handler->display->display_options['fields']['created']['table'] = 'node';
    $handler->display->display_options['fields']['created']['field'] = 'created';
    $handler->display->display_options['fields']['created']['label'] = '';
    $handler->display->display_options['fields']['created']['exclude'] = TRUE;
    $handler->display->display_options['fields']['created']['element_label_colon'] = FALSE;

    // Source Information.
    $handler->display->display_options['fields']['sonr_webmining_source_information']['id'] = 'sonr_webmining_source_information';
    $handler->display->display_options['fields']['sonr_webmining_source_information']['table'] = 'views';
    $handler->display->display_options['fields']['sonr_webmining_source_information']['field'] = 'nothing';
    $handler->display->display_options['fields']['sonr_webmining_source_information']['label'] = '';
    $handler->display->display_options['fields']['sonr_webmining_source_information']['alter']['text'] = '[created] | [sonr_webmining_source]';

    /* Field: Content: Title */
    $handler->display->display_options['fields']['title']['id'] = 'title';
    $handler->display->display_options['fields']['title']['table'] = 'node';
    $handler->display->display_options['fields']['title']['field'] = 'title';
    $handler->display->display_options['fields']['title']['label'] = '';
    $handler->display->display_options['fields']['title']['link_to_node'] = FALSE;
    $handler->display->display_options['fields']['title']['alter']['make_link'] = TRUE;
    $handler->display->display_options['fields']['title']['alter']['target'] = '_blank';
    $handler->display->display_options['fields']['title']['alter']['path'] = '[sonr_webmining_path]';
    $handler->display->display_options['fields']['title']['alter']['max_length'] = '200';
    $handler->display->display_options['fields']['title']['alter']['trim'] = TRUE;

    // Sentiment results.
    if ($this->config['show_sentiment']) {
      $handler->display->display_options['fields']['sonr_show_sentiment']['id'] = 'sonr_show_sentiment';
      $handler->display->display_options['fields']['sonr_show_sentiment']['table'] = 'field_data_body';
      $handler->display->display_options['fields']['sonr_show_sentiment']['field'] = 'body';
      $handler->display->display_options['fields']['sonr_show_sentiment']['label'] = '';
    }

    /* Field: Content: Body */
    $handler->display->display_options['fields']['body']['id'] = 'body';
    $handler->display->display_options['fields']['body']['table'] = 'field_data_body';
    $handler->display->display_options['fields']['body']['field'] = 'body';
    $handler->display->display_options['fields']['body']['label'] = '';
    $handler->display->display_options['fields']['body']['alter']['trim'] = TRUE;
    $handler->display->display_options['fields']['body']['alter']['html'] = TRUE;
    $handler->display->display_options['fields']['body']['alter']['word_boundary'] = TRUE;
    $handler->display->display_options['fields']['body']['alter']['ellipsis'] = TRUE;

    // Tags.
    if ($this->config['show_tags']) {
      $handler->display->display_options['fields']['sonr_webmining_tags']['id'] = 'sonr_webmining_tags';
      $handler->display->display_options['fields']['sonr_webmining_tags']['table'] = 'field_data_body';
      $handler->display->display_options['fields']['sonr_webmining_tags']['field'] = 'body';
      $handler->display->display_options['fields']['sonr_webmining_tags']['label'] = t('Tags');
      $handler->display->display_options['fields']['sonr_webmining_tags']['hide_empty'] = TRUE;
    }

    // Similar results (more like this).
    if ($this->config['show_similar'] && (!isset($variables['only_documents']) || !$variables['only_documents'])) {
      $handler->display->display_options['fields']['sonr_more_like_this']['id'] = 'sonr_more_like_this';
      $handler->display->display_options['fields']['sonr_more_like_this']['table'] = 'field_data_body';
      $handler->display->display_options['fields']['sonr_more_like_this']['field'] = 'body';
      $handler->display->display_options['fields']['sonr_more_like_this']['label'] = '';
    }

    /* No results behavior: Global: Text area */
    if (!isset($variables['only_documents']) || !$variables['only_documents']) {
      $note = t("Your search doesn't match with any content, please use another search term or refine your filters!");
    }
    else {
      $note = t('No similar documents found!');
    }
    $empty_content = '<div class="view-header">' . $header_content . '</div>';
    $empty_content .= '<div class="view-content note">' . $note . '</div>';
    $handler->display->display_options['empty']['area']['id'] = 'area';
    $handler->display->display_options['empty']['area']['table'] = 'views';
    $handler->display->display_options['empty']['area']['field'] = 'area';
    $handler->display->display_options['empty']['area']['label'] = 'No search result';
    $handler->display->display_options['empty']['area']['empty'] = TRUE;
    $handler->display->display_options['empty']['area']['content'] = $empty_content;
    $handler->display->display_options['empty']['area']['format'] = 'full_html';

    // Add the results.
    $view->result = array();
    $i = 0;
    if (is_array($this->result) && isset($this->result['results'])) {
      foreach ($this->result['results'] as $item) {
        drupal_alter('sonr_webmining_list_item', $item, $this->config, $this->sonrApi);

        if (isset($item['customAttributes'][SemanticConnectorSonrApi::ATTR_SOURCE])) {
          $type = $item['customAttributes'][SemanticConnectorSonrApi::ATTR_SOURCE];
          $link = $item['link'];
        }
        elseif (isset($item['customAttributes'][SemanticConnectorSonrApi::ATTR_CONTENT_TYPE])) {
          $type = $item['customAttributes'][SemanticConnectorSonrApi::ATTR_CONTENT_TYPE][0] . ' (' . variable_get('site_name') . ')';
          $link = str_replace($base_url . '/', '', $item['link']);
        }
        else {
          $type = variable_get('site_name');
          $link = $item['link'];
        }
        $result = new stdClass();
        $result->nid = $link;
        $result->node_type = $type;

        // Newer PoolParty GraphSearch versions always deliver arrays.
        if (is_array($result->node_type)) {
          $result->node_type = reset($result->node_type);
        }

        $result->node_created = round($item['date'] / 1000);
        $result->node_title = trim($item['title']);

        $field_body = array(
          array(
            'rendered' => array(
              '#prefix' => '<p>',
              '#suffix' => '</p>',
              '#markup' => views_trim_text(
                array(
                  'max_length' => $this->config['summary_max_chars'],
                  'word_boundary' => $handler->display->display_options['fields']['body']['alter']['word_boundary'],
                  'ellipsis' => $handler->display->display_options['fields']['body']['alter']['ellipsis'],
                  'html' => $handler->display->display_options['fields']['body']['alter']['html'],
                ),
                $item['description']),
              '#access' => TRUE,
            ),
          ),
        );
        $result->field_body = $field_body;

        // Add the dynamic sentiments.
        $result->field_sonr_show_sentiment = array();
        $attr_sentiment = SemanticConnectorSonrApi::ATTR_SENTIMENT;
        if ($this->config['show_sentiment'] && isset($item['customAttributes'][$attr_sentiment])) {

          $sentiment_value_negative = "";
          $sentiment_value_zero = "";
          $sentiment_value_positive = "";
          $sentiment_value_ceil = ceil(100 * $item['customAttributes'][$attr_sentiment]);

          if ($item['customAttributes'][$attr_sentiment] < 0) {
            $sentiment_value_negative = 'style="width:' . (-1 * 50) * $item['customAttributes'][$attr_sentiment] . '%"';
          }
          if ($item['customAttributes'][$attr_sentiment] == 0 || $item['customAttributes'][$attr_sentiment] == 'NaN') {
            $sentiment_value_zero = 'style="width:0%"';
          }
          if ($item['customAttributes'][$attr_sentiment] > 0) {
            $sentiment_value_positive = 'style="width:' . (1 * 50) * $item['customAttributes'][$attr_sentiment] . '%"';
          }

          $result->field_sonr_show_sentiment = array(
            array(
              'rendered' => array(
                '#markup' => '
                  <div>
                    <div class="sentimentColorRange">
                      <div class="rating">
                        <span class="bar">
                          <span class="wrapper">
                              <span class="negative" ' . $sentiment_value_negative . '></span>
                              <span class="zero" ' . $sentiment_value_zero . '></span>
                              <span class="positive" ' . $sentiment_value_positive . '></span>
                          </span>
                        </span>
                      </div>
                    </div>
                    <div class="sentimentValue">' . $sentiment_value_ceil . '%</div>
                    <div style="clear:both;"></div>
                  </div>
                ',
              ),
            ),
          );
        }

        // Add the tags.
        $result->field_sonr_webmining_tags = array();
        if ($this->config['show_tags'] && isset($item['customAttributes'][SemanticConnectorSonrApi::ATTR_ALL_CONCEPTS])) {
          if (isset($item['customAttributes'][SemanticConnectorSonrApi::ATTR_ALL_CONCEPTS]['prefLabel'])) {
            $item['customAttributes'][SemanticConnectorSonrApi::ATTR_ALL_CONCEPTS] = array($item['customAttributes'][SemanticConnectorSonrApi::ATTR_ALL_CONCEPTS]);
          }

          // Sort the tags by their prefLabels.
          $sorted_tags = array();
          foreach ($item['customAttributes'][SemanticConnectorSonrApi::ATTR_ALL_CONCEPTS] as $concept) {
            $sorted_tags[$concept['prefLabel']] = $concept;
          }
          uksort($sorted_tags, 'strcasecmp');

          // Choose which tags to show right away and which tags to hide.
          $tags_array = array(
            'show' => array(),
            'hide' => array(),
          );
          foreach ($sorted_tags as $concept) {
            $show_type = (empty($this->config['tags_max_items']) || count($tags_array['show']) < $this->config['tags_max_items']) ? 'show' : 'hide';
            $tags_array[$show_type][] = array(
              'uri' => (isset($concept['uri']) && !empty($concept['uri']) ? $concept['uri'] : ''),
              'html' => $concept['prefLabel'],
            );
          }

          // Get the project ID required for theming the tags.
          $connection_config = $this->connection->getConfig();
          $sonr_config = $connection_config['sonr_configuration'];
          $project_id = $this->search_space_id;
          if (is_array($sonr_config)) {
            if (version_compare($sonr_config['version'], '6.1', '>=')) {
              $search_spaces = semantic_connector_get_graphsearch_search_spaces($sonr_config);
              if (isset($search_spaces[$this->search_space_id])) {
                // A single project is connected with the search space.
                if (count($search_spaces[$this->search_space_id]['project_ids']) == 1) {
                  $project_id = $search_spaces[$this->search_space_id]['project_ids'][0];
                }
                // More than one project is connected with the search space
                // --> we can't provide a project ID.
                else {
                  $project_id = '';
                }
              }
            }
          }

          // Theme the tags.
          if (!empty($tags_array['hide'])) {
            $tags = '<span class="tags-show">';
            $tags .= semantic_connector_theme_concepts($tags_array['show'], $this->connection->getID(), $project_id) . ', <span class="tags-more" title="Show all tags">...</span>';
            $tags .= '<span class="tags-rest element-hidden">' . semantic_connector_theme_concepts($tags_array['hide'], $this->connection->getID(), $project_id) . '</span>';
            $tags .= '</span>';
          }
          else {
            $tags = semantic_connector_theme_concepts($tags_array['show'], $this->connection->getID(), $project_id);
          }

          $result->field_sonr_webmining_tags = array(
            array(
              'rendered' => array(
                '#markup' => $tags,
              ),
            ),
          );
        }

        // Add the "more like this"-button.
        $result->field_sonr_more_like_this = array();
        if ($this->config['show_similar'] && (!isset($variables['only_documents']) || !$variables['only_documents'])) {
          $result->field_sonr_more_like_this = array(
            array(
              'rendered' => array(
                '#markup' => '
                  <div id="similar-docs-' . $this->result['request']['start'] . '-' . $i++ . '" class="similar-docs">
                    <div class="similar-more" data-uri="' . $item['id'] . '">' . t('more like this') . '</div>
                    <div class="similar-content"></div>
                  </div>
                ',
              ),
            ),
          );
        }

        $view->result[] = $result;
      }
    }

    $view->build();
    $view->pre_execute();

    $vars = array(
      '#theme' => 'views_view',
      '#view' => $view,
      '#theme_wrappers' => array('views_container'),
      '#attributes' => array('data-id' => $this->id),
    );

    drupal_add_js(drupal_get_path('module', 'sonr_webmining') . '/js/sonr_webmining.js');
    drupal_add_js(array('sonr_webmining' => array('items_per_page' => $this->config['items_per_page'])), 'setting');

    if ($this->config['use_css_file']) {
      drupal_add_css(drupal_get_path('module', 'sonr_webmining') . '/css/sonr_webmining.css');
    }
    else {
      drupal_add_css(drupal_get_path('module', 'sonr_webmining') . '/css/sonr_webmining_simple.css');
    }

    return drupal_render($vars);
  }

  /**
   * Theme PoolParty GraphSearch results as an RSS feed.
   *
   * TODO: Use the view object to create the RSS Feed.
   *
   * @param array $variables
   *   Array of variables
   *
   * @return string
   *   The rendered content
   */
  protected function themeAsRSSfeed($variables = array()) {
    global $base_url, $language_content;

    $items = '';
    $trim_config = array(
      'max_length' => 500,
      'word_boundary' => TRUE,
      'ellipsis' => TRUE,
    );
    if (is_array($this->result) && isset($this->result['results'])) {
      foreach ($this->result['results'] as $item) {
        $args = array(
          'pubDate' => gmdate(DATE_RSS, $item['date'] / 1000),
        );
        $items .= format_rss_item($item['title'], $item['link'], views_trim_text($trim_config, $item['description']), $args);
      }
    }

    $channel = array();
    $namespaces = array('xmlns:dc' => 'http://purl.org/dc/elements/1.1/');

    $channel_defaults = array(
      'version' => '2.0',
      'title' => t('PoolParty GraphSearch'),
      'link' => $base_url,
      'description' => t('Customized Feed'),
      'language' => $language_content->language,
      'generator' => 'http://drupal.org/',
    );
    $channel_extras = array_diff_key($channel, $channel_defaults);
    $channel = array_merge($channel_defaults, $channel);

    $output = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
    $output .= "<rss version=\"" . $channel["version"] . "\" xml:base=\"" . $base_url . "\"" . drupal_attributes($namespaces) . ">\n";
    $output .= format_rss_channel($channel['title'], $channel['link'], $channel['description'], $items, $channel['language'], $channel_extras);
    $output .= "</rss>\n";

    drupal_add_http_header('Content-Type', 'application/rss+xml; charset=utf-8');

    return $output;
  }

  /**
   * Add filters to an existing Drupal-form.
   *
   * @param array $form
   *   The form, where the filters have to be added.
   * @param array $form_state
   *   The state of the form
   *
   * @return array
   *   The manipulated form
   */
  public function addFiltersToForm($form, &$form_state) {

    $form['sw_filter_container'] = array(
      '#type' => 'container',
      '#prefix' => '<div class="views-exposed-form">',
      '#suffix' => '</div>',
      '#attributes' => array(
        'class' => array('views-exposed-widgets clearfix'),
      ),
    );

    $filter_list = array();
    foreach ($this->filters as $filter) {
      if ($filter->field == 'date-from' || $filter->field == 'date-to') {
        $filter_list[$filter->field] = $filter->value;
      }
      else {
        $filter_list[] = $filter->value;
      }
    }

    foreach ($this->config['components_order'] as $component_id) {
      switch ($component_id) {
        // Add the facets.
        case 'facets':
          if (is_array($this->result) && isset($this->result['facetList'])) {
            $facet_list = $this->result['facetList'];
            $facet_nr = 1;
            foreach ($facet_list as $item) {
              $fieldset_name = 'sw_fieldset_' . $item['field'];
              $tag_id = strtolower(str_replace('_', '-', $item['field']));
              $prefix = '
                <div id="edit-' . $tag_id . '-wrapper" class="views-exposed-widget views-widget-filter-' . $tag_id . '">
                  <div class="views-widget">
                    <div class="form-item form-type-select form-item-' . $tag_id . '">';
              $form['sw_filter_container'][$fieldset_name] = array(
                '#type' => 'fieldset',
                '#prefix' => $prefix,
                '#suffix' => '</div></div></div>',
                '#title' => $item['label'],
                '#collapsible' => TRUE,
                '#attributes' => array(
                  'class' => array('bef-select-as-checkboxes-fieldset'),
                ),
              );
              $form['sw_filter_container'][$fieldset_name][$item['field']] = array(
                '#prefix' => '<div id="edit-' . $tag_id . '" class="form-checkboxes">',
                '#suffix' => '</div>',
                '#tree' => TRUE,
              );

              $selected = $this->createCheckboxes($form['sw_filter_container'][$fieldset_name][$item['field']], $item['field'], $item['facets'], $filter_list);
              $form['sw_filter_container'][$fieldset_name]['#collapsed'] = (($facet_nr == 1 && empty($filter_list)) || $selected) ? FALSE : TRUE;
              $facet_nr++;
            }

            // Add the facet to the facet box if the URL parameter uri is given.
            $facet_box = array();
            if (isset($_GET['uri']) || isset($_GET['search'])) {
              foreach ($this->filters as $filter) {
                switch ($filter->field) {
                  case SemanticConnectorSonrApi::ATTR_CONTENT:
                    $facet_box[] = array(
                      'type' => 'free-term',
                      'field' => SemanticConnectorSonrApi::ATTR_CONTENT,
                      'value' => $filter->value,
                      'label' => $filter->value,
                    );
                    break;

                  default:
                    $facet_box[] = array(
                      'type' => 'concept',
                      'field' => $filter->field,
                      'value' => $filter->value,
                      'label' => $filter->label,
                    );
                    break;
                }
              }
            }
            drupal_add_js(array('sonr_webmining' => array('facet_box' => $facet_box)), 'setting');
          }
          break;

        // Add the date filter.
        case 'time':
          if ($this->config['time_filter'] == 'from_to_textfields') {
            $form['sw_filter_container']['date_from'] = array(
              '#type' => 'date_popup',
              '#title' => t('from'),
              '#default_value' => isset($filter_list['date-from']) ? $filter_list['date-from'] : '',
              '#date_format' => 'Y-m-d',
              '#date_label_position' => 'none',
              '#attributes' => array('data-field' => 'date-from'),
              '#date_year_range' => '-10:+1',
            );
            $form['sw_filter_container']['date_to'] = array(
              '#type' => 'date_popup',
              '#title' => t('to'),
              '#default_value' => isset($filter_list['date-to']) ? $filter_list['date-to'] : '',
              '#date_format' => 'Y-m-d',
              '#date_label_position' => 'none',
              '#attributes' => array('data-field' => 'date-to'),
              '#date_year_range' => '-10:+1',
            );
          }
          elseif ($this->config['time_filter'] == 'range_selection') {
            $dates = array(
              'no time limit' => date("Y-m-d", 0),
              'last day' => date("Y-m-d", time() - 60 * 60 * 24),
              'last week' => date("Y-m-d", time() - 60 * 60 * 24 * 7),
              'last month' => date("Y-m-d", time() - 60 * 60 * 24 * 31),
              'last year' => date("Y-m-d", time() - 60 * 60 * 24 * 365),
            );
            $date_options = array();
            foreach ($dates as $date_key => $date) {
              $date_options[$date] = $date_key;
            }
            if (!empty($this->config['time_filter_years'])) {
              $this_year = date('Y');
              for ($year = $this_year; $year >= $this->config['time_filter_years']; $year--) {
                $date_options[$year] = 'in ' . $year;
              }
            }
            $form['sw_filter_container']['date_from'] = array(
              '#type' => 'select',
              '#prefix' => '<div id="edit-date-from-wrapper" class="views-exposed-widget views-widget-filter-date-from"><div class="views-widget">',
              '#suffix' => '</div></div>',
              '#title' => t('Time range'),
              '#options' => $date_options,
              '#default_value' => isset($filter_list['date-from']) ? $filter_list['date-from'] : $dates['no time limit'],
              '#attributes' => array(
                'data-field' => 'date-from',
                'name' => 'date_from[date]'
              ),
            );
          }
          break;

        // Add the reset button.
        case 'reset':
          $form['sw_filter_container']['reset'] = array(
            '#prefix' => '<div class="views-exposed-widget views-reset-button">',
            '#suffix' => '</div>',
            '#type'   => 'submit',
            '#value'  => 'Reset',
            '#name'   => 'op',
            '#attributes' => array('class' => array('filter-reset')),
          );
          break;

        // Add the trends as a chart if required.
        case 'trends':
          if ($this->config['add_trends']) {
            $this->addTrendsToForm($form, $form_state);
          }
          break;
      }
    }

    // Add additional styling.
    if (!$this->config['separate_blocks']) {
      $form['#attributes']['class'][] = 'sonr-webmining-area-left';
    }

    return $form;
  }

  /**
   * Creates the checkboxes for the search facets.
   *
   * @param array $form
   *   The facet filter form.
   * @param string $field
   *   The ID for the facet field.
   * @param array $facets
   *   The facets for the facet field.
   * @param array $filter_list
   *   The list of selected filters.
   * @param int $depth
   *   The depth of the tree for the tree view mode.
   *
   * @return bool
   *   TRUE if one of the filters are found in the facet field, otherwise FALSE.
   */
  protected function createCheckboxes(&$form, $field, $facets, $filter_list, $depth=0) {
    $found = FALSE;
    foreach ($facets as $facet) {
      $default_value = FALSE;
      if (in_array($facet['value'], $filter_list)) {
        $found = TRUE;
        $default_value = TRUE;
      }
      $form_key = $facet['value'] . '_' . $depth;
      $form[$form_key] = array(
        '#type' => 'checkbox',
        '#title' => $facet['label'] . ' (' . $facet['count'] . ')',
        '#default_value' => $default_value,
        '#return_value' => $facet['value'],
        '#attributes' => array(
          'data-field' => $field,
        ),
      );
      if (!empty($facet['children'])) {
        $form[$form_key . '-tree-open'] = array(
          '#markup' => '<div class="tree-depth">',
        );
        if ($this->createCheckboxes($form, $field, $facet['children'], $filter_list, $depth+1)) {
          $form[$form_key]['#suffix'] = '<div class="tree-open-close"></div>';
          $form[$form_key . '-tree-open']['#markup'] = '<div class="tree-depth" style="display: block;">';
          $found = TRUE;
        }
        else {
          $form[$form_key]['#suffix'] = '<div class="tree-open-close collapsed"></div>';
        }
        $form[$form_key . '-tree-close'] = array(
          '#markup' => '</div>',
        );
      }
    }

    return $found;
  }

  /**
   * Add search bar with text field and buttons to an existing Drupal-form.
   *
   * @param array $form
   *   The form, where the filters have to be added.
   * @param array $form_state
   *   The state of the form
   *
   * @return array
   *   The manipulated form
   */
  public function addSearchBarToForm($form, &$form_state) {
    $form['search_string'] = array(
      '#prefix' => '<div class="sonr-webmining-search-bar">',
      '#type' => 'textfield',
      '#title' => t('Search'),
      '#default_value' => '',
      '#attributes' => array('placeholder' => t($this->config['placeholder']), 'class' => array('form-autocomplete')),
    );

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Search'),
      '#attributes' => array('class' => array('search-bar-submit')),
    );

    $form['reset'] = array(
      '#suffix' => '</div><div class="clearBoth"></div>',
      '#type'   => 'submit',
      '#value'  => 'Reset',
      '#name'   => 'op',
      '#attributes' => array('class' => array('search-bar-reset')),
    );

    return $form;
  }

  /**
   * Add the facet box for the selected concepts and free terms.
   *
   * @param array $form
   *   The form, where the filters have to be added.
   * @param array $form_state
   *   The state of the form
   *
   * @return array
   *   The manipulated form
   */
  public function addFacetBoxToForm($form, &$form_state) {
    $form['facet_box_form'] = array(
      '#type' => 'container',
      '#prefix' => '<div id="facet-container"><div class="facets-empty">' . t('No active filters') . '</div><div class="facet-box"></div>',
      '#suffix' => '<div class="search-filters element-hidden"></div></div>',
      '#attributes' => array('class' => array('element-hidden', 'facet-box-form')),
    );

    if (user_is_logged_in()) {
      $form['facet_box_form']['title'] = array(
        '#prefix' => '<div class="messages error element-hidden"></div>',
        '#type' => 'textfield',
        '#title' => t('Title'),
        '#required' => TRUE,
      );
      $options = drupal_map_assoc(array('', 'daily', 'weekly', 'monthly'));
      $form['facet_box_form']['time_interval'] = array(
        '#type' => 'select',
        '#title' => t('Time interval'),
        '#description' => t('Please select a time interval if you want a personalized email alert to the selected filter; if not then leave it empty.'),
        '#options' => $options,
      );
      $form['facet_box_form']['save'] = array(
        '#type'   => 'submit',
        '#value'  => 'Save search filter',
        '#attributes' => array('class' => array('facet-box-form-submit')),
      );

      $form['filter_buttons'] = array(
        '#markup' => '<div id="filter-operations" class="filter-operations"><a class="add element-hidden">' . t('Add to my filters') . '</a><a class="show">' . t('Show my filters') . '</a></div>',
      );
    }

    return $form;
  }

  /**
   * Add a chart of trends to an existing Drupal-form.
   *
   * @param array $form
   *   The form, where the filters have to be added.
   * @param array $form_state
   *   The state of the form
   */
  protected function addTrendsToForm(&$form, &$form_state) {
    if (module_exists('sonr_webmining_trends') && !empty($this->filters)) {
      $selected_facets = array();
      $unimportant_facet_ids = array(
        SemanticConnectorSonrApi::ATTR_SOURCE,
        SemanticConnectorSonrApi::ATTR_REGIONS,
        SemanticConnectorSonrApi::ATTR_CONTENT,
        SemanticConnectorSonrApi::ATTR_CONTENT_TYPE,
        'type',
        'date-from',
      );
      foreach ($this->filters as $filter) {
        if (!in_array($filter->field, $unimportant_facet_ids)) {
          $selected_facets[] = $filter->value;
        }
      }
      if (!empty($selected_facets)) {
        $sonr_api = $this->sonrApi;
        $trends_graph = new SonrWebminingTrends($sonr_api, $selected_facets, $this->getSearchSpaceId());

        $form['sw_filter_container']['trends'] = array(
          '#type'   => 'item',
          '#prefix' => '<div id="edit-trends-wrapper" class="views-exposed-widget views-widget-filter-trends"><div class="views-widget">',
          '#suffix' => '</div></div>',
          '#markup' => $trends_graph->theme('chart', array(
            'width' => '100%',
          )),
        );
      }
    }
  }

  /**
   * Add search bar with text field to an existing block.
   *
   * @param array $form
   *   The form, where the filters have to be added.
   * @param array $form_state
   *   The state of the form
   *
   * @return array
   *   The manipulated form
   */
  public function addBlockSearchBarToForm($form, &$form_state) {
    $form['block_search_string'] = array(
      '#prefix' => '<div class="sonr-webmining-block-search-bar">',
      '#suffix' => '</div>',
      '#type' => 'textfield',
      '#default_value' => '',
      '#attributes' => array(
        'placeholder' => t($this->config['placeholder']),
        'class' => array('form-autocomplete'),
        'data-autocomplete-path' => 'sonr-webmining/autocomplete/' . $this->id . '/' . $this->config['ac_max_suggestions'],
        'data-page-path' => sonr_webmining_get_page_path($this->getId()),
        'data-min-chars' => $this->config['ac_min_chars'],
        'data-add_matching_label' => $this->config['ac_add_matching_label'],
        'data-add_context' => $this->config['ac_add_context'],
        'data-search-type' => $this->config['search_type'],
      ),
    );

    return $form;
  }

  /**
   * Sorts a list of concept URIs according the order list.
   *
   * @param array $concept_uris
   *   The list of concept URIs.
   * @param array $facet_order
   *   The list according to which the concept URIs are to be sorted.
   * @param array $facet_map
   *   A map between the facet ID and its URI.
   *
   * @return array
   *   The sorted concept URIs.
   */
  protected function sortConceptUris($concept_uris, $facet_order, $facet_map) {
    $map = array();
    foreach ($concept_uris as $uri) {
      if (isset($facet_map[$uri])) {
        $map[$facet_map[$uri]] = $uri;
      }
    }

    return array_replace(array_intersect_key($facet_order, $map), $map);
  }

  /**
   * Clips the tree by max items and depth.
   * @param array $tree
   * @param int $max_items
   * @param int $depth
   * @return array
   */
  protected function clipTree($tree, $max_items, $depth) {
    if ($depth < 0 || empty($tree)) {
      return array();
    }

    foreach ($tree as $key => &$branch) {
      if (!empty($branch['children'])) {
        $branch['children'] = array_slice($branch['children'], 0, $max_items);
        $branch['children'] = $this->clipTree($branch['children'], $max_items, $depth-1);
      }
    }

    return $tree;
  }

}
